@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 """
 PYIDSERVER.PY
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -14,8 +14,8 @@ about it.
 import sys
 import socket
 
-from plib.stdlib.options import parse_options
-from plib.utils.chatgen import chat_replies
+from plib.utils.chatgen import chat_replies as _chat_replies
+
 
 def do_output(fileobj, s, linesep=True):
     if linesep:
@@ -24,7 +24,18 @@ def do_output(fileobj, s, linesep=True):
         fileobj.write(s)
     fileobj.flush()
 
-run_callback = None # used by pyidserver-gui
+
+class chat_replies(_chat_replies):
+    # Adds initial output when connection is made
+    
+    def __init__(self, addr, items, fileobj, callback=None):
+        _chat_replies.__init__(self, addr, items, callback)
+        self.fileobj = fileobj
+    
+    def on_connect(self):
+        do_output(self.fileobj, "Connected ...")
+        do_output(self.fileobj, "Server returned the following:")
+
 
 PROTO_DEFAULT = 'http'
 
@@ -38,32 +49,8 @@ protocols = {
     'pop': (110, quitmsgs),
     'smtp': (25, quitmsgs) }
 
+
 def run_idserver(fileobj, arg, dns_only, protocol, portnum):
-    """
-    Query server and write results to fileobj.
-    
-    Server argument 'arg' should be a URL string, either an IP address
-    or a host name. The URL may include a protocol specifier at the
-    start (e.g., http://), and may include a port specifier at the
-    end (e.g., :80). A trailing slash, '/', in the URL, and anything
-    after it, are treated as a path specifier and ignored.
-    
-    If dns_only is true, only a DNS lookup is done; no connection is
-    actually made to the server.
-    
-    The protocol should be one of the strings listed as keys in the
-    protocols dictionary above.
-    
-    The port number should be an integer giving the port number on
-    the server. (This parameter should only need to be used very
-    rarely; almost always the port number is determined by the
-    protocol as shown in the dictionary above.)
-    
-    The output file object 'fileobj' can be any file-like object
-    that has a write and a flush method. See pyidserver-gui.py for
-    an example of a file-like object that writes to a text control.
-    """
-    
     if '://' in arg:
         addrtype, arg = arg.split('://')
         if (addrtype in protocols):
@@ -73,23 +60,27 @@ def run_idserver(fileobj, arg, dns_only, protocol, portnum):
                     % (addrtype, protocol))
             protocol = addrtype
         elif addrtype:
-            do_output(fileobj, "URL includes incorrect protocol %s, ignoring."
+            do_output(fileobj,
+                "URL includes incorrect protocol %s, ignoring."
                 % addrtype)
     if '/' in arg:
         arg, path = arg.split('/')
         if path:
-            do_output(fileobj, "URL includes path after host name/address, ignoring.")
+            do_output(fileobj, "URL includes path, ignoring.")
     if ':' in arg:
         arg, portstr = arg.split(':')
         try:
             p = int(portstr)
             if p != portnum:
                 if portnum != 0:
-                    do_output(fileobj, "URL includes port %d, ignoring specified port %d."
+                    do_output(fileobj,
+                        "URL includes port %d, ignoring specified port %d."
                         % (p, portnum))
                 portnum = p
         except ValueError:
-            do_output(fileobj, "URL includes invalid port %s, ignoring." % portstr)
+            do_output(fileobj,
+                "URL includes invalid port %s, ignoring." %
+                portstr)
     
     if dns_only:
         do_output(fileobj, "Doing DNS lookup on %s ..." % arg)
@@ -120,28 +111,82 @@ def run_idserver(fileobj, arg, dns_only, protocol, portnum):
         do_output(fileobj, "IP address for %s is %s." % (url, ipaddr))
     
     if not dns_only:
-        replied = False
-        do_output(fileobj, "Contacting %s%s%s ..." % (arg, proto_msg, port_msg))
-        for reply in chat_replies((url, portnum), proto_items, run_callback):
-            if not replied:
-                replied = True
-                do_output(fileobj, "Connected ...")
-                do_output(fileobj, "Server returned the following:")
+        do_output(fileobj,
+            "Contacting %s%s%s ..." %
+                (arg, proto_msg, port_msg))
+        for reply in chat_replies((url, portnum), proto_items, fileobj):
             do_output(fileobj, reply, False)
         do_output(fileobj, "Connection closed.")
 
-def run_main(outfile, arg, errfile=None, dns_only=False, protocol="", portnum=0):
+
+def run_main(outfile, arg, errfile=None,
+        dns_only=False, protocol="", portnum=0):
+    """Query server and write results to a file-like object.
+    
+    This is the intended external API for pyidserver; it wraps the
+    ``run_idserver`` function, which does the work, with reasonable
+    error handling and diagnostic output.
+    
+    The purpose of pyidserver is to query an internet server for
+    basic information, and output it to the user. It does not actually
+    "speak" any of the specific protocols for which it will query a
+    server; it relies on the fact that most servers return some sort
+    of informational "greeting" when a client connects to them, and
+    the information it outputs is taken from such greetings.
+    
+    In the case of HTTP servers, a request must first be sent for the
+    server to return any information (a HEAD request is used for this
+    purpose). In the case of IMAP servers, an additional request after
+    the first greeting (A1 CAPABILITY) is used to elicit additional
+    information.
+    
+    In all cases where the session with the server is supposed to be
+    explicitly terminated (all protocols supported except FTP),
+    pyidserver does the termination when it is finished.
+    
+    Arguments:
+    
+    - ``outfile``: the file-like object for output (actually it
+      can be anything that has ``write`` and ``flush`` methods).
+      Defaults to standard output.
+    
+    - ``arg``: a URL string (either an IP address or a host name).
+      May include a protocol specifier at the start (e.g., http://),
+      and may include a port specifier at the end (e.g., :80). A
+      trailing slash, '/', in the URL, and anything after it, are
+      treated as a path specifier and ignored.
+    
+    - ``errfile``: a file-like object for error output (actually it
+      can be anything with a ``write`` method). Defaults to the same
+      object as ``outfile``.
+    
+    - ``dns_only``: If true, only a DNS lookup is done; no connection
+      is actually made to the server.
+    
+    - ``protocol: one of the strings listed as keys in the
+      ``protocols`` dictionary above (the default, if nothing is
+      passed, is 'http').
+    
+    - ``portnum``: an integer giving the port number on the server.
+      (This parameter should only need to be used very rarely;
+      almost always the port number is determined by the protocol
+      as shown in the dictionary above.)
+    """
+    
     if errfile is None:
         errfile = outfile
     try:
         run_idserver(outfile, arg, dns_only, protocol, portnum)
     except ValueError:
         errfile.write("%s\n" % str(sys.exc_info()[1]))
-    except (ValueError, socket.error, socket.herror, socket.gaierror, socket.timeout):
+    except (socket.error, socket.herror, socket.gaierror, socket.timeout):
         exc_type, exc_value, _ = sys.exc_info()
         errfile.write("%s %s\n" % tuple(map(str, (exc_type, exc_value))))
 
+
 if __name__ == '__main__':
+    from plib.stdlib.options import parse_options
+    
     _, def_dns, def_proto, def_port = run_main.func_defaults
     optlist = (
         ("-l", "--lookup", { 'action': "store_true",
@@ -157,4 +202,5 @@ if __name__ == '__main__':
     arglist = ["URL"]
     
     opts, args = parse_options(optlist, arglist)
-    run_main(sys.stdout, args[0], sys.stderr, opts.dns_only, opts.protocol, opts.portnum)
+    run_main(sys.stdout, args[0], sys.stderr,
+        opts.dns_only, opts.protocol, opts.portnum)
