@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 """
 SCRIPS.PY
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -22,8 +22,10 @@ from plib.classes import TokenConverter
 scripsdirname = ".scrips"
 scripsdatname = "scrips.dat"
 
+
 # This is dynamic so changing the above globals will change the
 # file name retrieved at run time
+
 def scripsdatfile():
     # First make sure the directory exists (this will allow the
     # file to be created if it doesn't exist)
@@ -33,13 +35,18 @@ def scripsdatfile():
         os.mkdir(dirname)
     return os.path.join(dirname, scripsdatname)
 
+
+TMPL = "Rx #%s for %s: last filled on %s for %s days, %s refills remaining%s."
+
+
 class Scrip(TokenConverter):
     
     turnaround = 5 # lead time for normal refills
     leadtime = 5 # additional time if scrip must be renewed
     
     # This makes data conversion to/from string tokens easier
-    converters = [('name', str),
+    converters = [
+        ('name', str),
         ('rxnum', str),
         ('filldate', strtodate),
         ('days', int),
@@ -65,15 +72,12 @@ class Scrip(TokenConverter):
             return ""
     
     def __str__(self):
-        return "Rx #%s for %s: last filled on %s for %s days, %s refills remaining%s." % \
-            (self.rxnum, self.name, self.filldate, self.days, self.refills, self._duestr())
+        return TMPL % \
+            (self.rxnum, self.name, self.filldate, self.days, self.refills,
+                self._duestr())
 
-# This is so that another program importing this as a module (e.g., scrips-edit)
-# can subclass Scrip above to add other capabilities, and have scriplist below
-# return a list of instances of the subclass.
-scripclass = Scrip
 
-def scriplist():
+def scriplist(scripclass=Scrip):
     l = []
     fname = scripsdatfile()
     if os.path.isfile(fname):
@@ -86,7 +90,10 @@ def scriplist():
             f.close()
     return l
 
-username = os.path.basename(os.getenv('HOME'))
+
+username = os.getenv('USER')
+if not username:
+    username = os.getenv('USERNAME')
 useraddr = "%s@localhost" % username
 optnames = [
     ('fromaddr', useraddr),
@@ -98,17 +105,20 @@ optnames = [
     ('username', ""),
     ('password', "") ]
 
+
 class ScripsIniFile(ini.PIniFile):
     _optionlist = [
-        ("email", [(optname, ini.INI_STRING, optdefault) for optname, optdefault in optnames]),
+        ("email", [(optname, ini.INI_STRING, optdefault)
+            for optname, optdefault in optnames]),
         ("headers", [("dict", ini.INI_STRING, "{}")]),
         ("pharmacy", [("name", ini.INI_STRING, "Pharmacy")]) ]
 
+
 inifile = ScripsIniFile("scrips")
 
+
 if __name__ == "__main__":
-    
-    from plib.stdlib import options
+    from plib.stdlib.options import parse_options
     
     optlist = (
         ("-d", "--display-only", { 'action': "store_true",
@@ -118,7 +128,7 @@ if __name__ == "__main__":
             'dest': "silent", 'default': False,
             'help': "send notification e-mail if scrip is due (default)" } )
         )
-    opts, args = options.parse_options(optlist)
+    opts, args = parse_options(optlist)
     
     if opts.silent:
         
@@ -151,12 +161,15 @@ if __name__ == "__main__":
                 msg['X-%s' % hname] = hvalue
             msg.set_type(inifile.email_typestr)
             msg.set_payload(mailstr(s), inifile.email_charsetstr)
-            server = smtplib.SMTP(inifile.email_serverstr, int(inifile.email_portnum))
+            server = smtplib.SMTP(
+                inifile.email_serverstr, int(inifile.email_portnum))
             server.set_debuglevel(1)
             if inifile.email_username and inifile.email_password:
                 server.starttls()
                 server.login(inifile.email_username, inifile.email_password)
-            server.sendmail(inifile.email_fromaddr, [inifile.email_toaddr], msg.as_string())
+            server.sendmail(
+                inifile.email_fromaddr, [inifile.email_toaddr],
+                msg.as_string())
             server.quit()
         
         def do_scrip(s):
