@@ -2,7 +2,7 @@
 """
 Module PTCPServer
 Sub-Package CLASSES of Package PLIB
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -16,22 +16,16 @@ from plib.stdlib.io.blocking import BaseRequestHandler, SocketServer
 
 from PServerBase import PServerBase
 
+
 class PTCPServer(PServerBase, SocketServer):
-    """
-    Generic forking TCP server class. Adds handling of the
-    ``terminate_sig`` flag to the ``keep_running`` check.
+    """Generic forking server class with enhanced common functionality.
+    
+    This generic forking server implements signal handling for controlled
+    termination, and log file functionality. The intent is to trap any
+    signal that might be used to indicate general 'program shutdown' as
+    opposed to some specific error condition (i.e., any signal where it
+    can be assumed that controlled shutdown of the Python interpreter
+    is possible).
     """
     
-    bind_addr = ("localhost", 9999)
     handler_class = BaseRequestHandler
-    
-    def __init__(self):
-        PServerBase.__init__(self)
-        SocketServer.__init__(self, self.bind_addr, self.handler_class)
-    
-    def server_close(self):
-        SocketServer.server_close(self)
-        PServerBase.server_close(self)
-    
-    def keep_running(self):
-        return SocketServer.keep_running(self) and (self.terminate_sig is None)
