@@ -2,7 +2,7 @@
 """
 Module SerialTelnet
 Sub-Package CLASSES of Package PLIB
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -17,11 +17,12 @@ import telnetlib
 
 from plib.classes import SerialPseudoSocket
 
+
 class SerialTelnet(telnetlib.Telnet):
-    """
-    Telnet client that can talk over a serial port as well as
-    a socket. Use the ``openserial`` method to open a serial
-    device, and ``open`` to open a socket. Does not allow any
+    """Telnet client that can use a serial port or a socket.
+    
+    Use the ``openserial`` method to open a serial device,
+    and ``open`` to open a socket. Does not allow any
     parameters to the constructor (so you have to explicitly
     call the appropriate opening method).
     """
