@@ -2,7 +2,7 @@
 """
 Module EDIT -- Python Editor Objects
 Sub-Package GUI of Package PLIB -- Python GUI Framework
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -24,17 +24,19 @@ import os
 from plib.gui.defs import *
 from plib.gui._gui import gui_toolkit
 
-raw_edit_toolkits = (GUI_QT, GUI_KDE, GUI_QT4, GUI_KDE4)
+raw_edit_toolkits = (GUI_QT, GUI_KDE)
 
 if gui_toolkit not in raw_edit_toolkits:
+    # All the editor methods take no arguments, so we need a
+    # wrapper to take them up
     def _editor_wrapper(target):
         def wrapper(*args, **kwargs):
             target()
         return wrapper
 
+
 class PEditor(object):
-    """
-    A base class to provide 'editor' type methods.
+    """Base class to provide 'editor' type methods.
     
     Can be used as a mixin or as a helper class; constructor sets up
     mainwidget field to point to appropriate widget for UI interface
@@ -79,50 +81,65 @@ class PEditor(object):
         'savesignals': 'save' }
     
     def __init__(self, mainwidget=None, control=None, data=None):
+        # Figure out the main widget
         if mainwidget is not None:
+            # Typically this will happen if we're a helper class
             self.mainwidget = mainwidget
         elif hasattr(self, '_parent') and (self._parent is not None):
+            # Typically this will happen if we're mixed in with a
+            # client widget class (e.g., in the ``scrips-edit`` example)
             self.mainwidget = self._parent
         else:
+            # This should only happen if we're mixed in with a top
+            # window class
             self.mainwidget = self
         if (self.mainwidget is not None):
             # This will serve for now to make sure main widget is
-            # a main window instance
+            # a top window instance
             if not hasattr(self.mainwidget, 'connectaction'):
                 self.mainwidget = None
         
+        # Figure out the control that is doing the "editing"
         if control is not None:
+            # Typically this will happen if we're a helper class
             self.control = control
+        elif self.mainwidget is self:
+            # If we're the main widget, the editing control should be
+            # our client widget (e.g., in the ``pnotepad`` example)
+            self.control = self.clientwidget
         else:
+            # This should only happen if we're actually mixed in with
+            # the control class
             self.control = self
+        
+        # The data should normally be initial data to be loaded by the
+        # control
         self.data = data
         
-        # flags to track state for button enable/disable
+        # Flags to track state for button enable/disable
         self._editable = False
         self._pending = False
         self._modified = False
         
-        # get pointer to toolbar update method
+        # Get pointer to toolbar update method
         if self.mainwidget is not None:
             self._updatefunc = self.mainwidget.updateactions
         else:
             self._updatefunc = None
         
-        # if we have a control and data, load data and populate control
+        # If we have a control and data, load data and populate control
         if (self.control is not None) and (self.data is not None):
             self.load()
         
-        # connect to mapped signals
+        # Connect to mapped signals
         for mapname, targetname in self.signalmaps.iteritems():
             self._setup_signals(mapname, targetname)
         
-        # connect main widget actions to our methods
+        # Connect main widget actions to our methods
         if self.mainwidget is not None:
-            self.mainwidget.connectaction(ACTION_EDIT, self.edit)
-            self.mainwidget.connectaction(ACTION_COMMIT, self.commit)
-            self.mainwidget.connectaction(ACTION_CANCEL, self.cancel)
+            self._connect_mainwidget()
         
-        # connect control actions to our methods; note that this must
+        # Connect control actions to our methods; note that this must
         # be separate from the signal mapping above because here we
         # need to preserve the signature of the control callback
         if self.control is not None:
@@ -145,16 +162,24 @@ class PEditor(object):
                 else:
                     source.setup_notify(signal, target)
     
+    def _connect_mainwidget(self):
+        self.mainwidget.connectaction(ACTION_EDIT, self.edit)
+        self.mainwidget.connectaction(ACTION_COMMIT, self.commit)
+        self.mainwidget.connectaction(ACTION_ROLLBACK, self.cancel)
+    
     def _connect_control(self):
-        """ Override in derived classes to connect to appropriate control signals. """
+        """Override in derived classes to connect actions to control signals.
+        """
         pass
     
     def _gettype(self):
-        """ Override in derived classes to customize file type name for display """
+        """Override in derived classes to customize displayed file type.
+        """
         return "Object"
     
     def _getname(self):
-        """ Override in derived classes to customize file name for display """
+        """Override in derived classes to customize displayed file name.
+        """
         return ""
     
     def _update(self):
@@ -190,86 +215,104 @@ class PEditor(object):
     modified = property(_get_modified, _set_modified)
     
     def _donew(self):
-        """ override in derived classes to create new file and populate widget """
+        """Override in derived classes to create new file and populate widget.
+        """
         pass
     
     def _doload(self):
-        """ override in derived classes to populate widget with file data """
+        """Override in derived classes to populate widget with file data.
+        """
         pass
     
     def _doedit(self):
-        """ override in derived classes to activate editor """
+        """Override in derived classes to activate editor.
+        """
         pass
     
     def _docommit(self):
-        """ override in derived classes to commit transaction """
+        """Override in derived classes to commit transaction.
+        """
         pass
     
     def _docancel(self):
-        """ override in derived classes to cancel transaction """
+        """Override in derived classes to cancel transaction.
+        """
         pass
     
     def _domodify(self):
-        """ override in derived classes to modify data """
+        """Override in derived classes to modify data.
+        """
         pass
     
     def _dosave(self):
-        """ override in derived classes to save data from widget """
+        """Override in derived classes to save data from widget.
+        """
         pass
     
     def _format_caption(self):
-        return self.captionformat % (self._gettype(), self.typesuffix, self._getname())
+        return self.captionformat % (
+            self._gettype(), self.typesuffix, self._getname())
     
     def switchname(self):
-        """ Call this method to update UI for new name """
+        """Call this method to update UI for new name.
+        """
         if self.mainwidget is not None:
             self.mainwidget.set_caption(self._format_caption())
     
     def new(self):
-        """ Call this method to open the editor with a new (unnamed) file """
+        """Call this method to open the editor with a new (unnamed) file.
+        """
         self._donew()
         self.pending = False
         self.modified = False
     
     def load(self):
-        """ Call this method to open the editor and load data """
+        """Call this method to open the editor and load data.
+        """
         self._doload()
         self.pending = False
         self.modified = False
     
     def edit(self):
-        """ Call this method to put editor into edit mode """
+        """Call this method to put editor into edit mode.
+        """
         if self.editable:
             self._doedit()
     
     def changed(self):
-        """ Call this method when a transaction starts """
+        """Call this method when a transaction starts.
+        """
         if self.editable:
             self.pending = True
     
     def commit(self):
-        """ Call this method to commit a transaction """
+        """Call this method to commit a transaction.
+        """
         self._docommit()
         self.pending = False
         self.modified = True
     
     def cancel(self):
-        """ Call this method to cancel a transaction """
+        """Call this method to cancel a transaction.
+        """
         self._docancel()
         self.pending = False
     
     def modify(self):
-        """ Call this method when the data is modified outside a transaction """
+        """Call this method when the data is modified outside a transaction.
+        """
         self._domodify()
         self.modified = True
     
     def save(self):
-        """ Call this method to save changes to storage """
+        """Call this method to save changes to storage.
+        """
         self._dosave()
         self.modified = False
     
     def querycommit(self):
-        """ ask user whether to commit uncommitted changes """
+        """Ask user whether to commit uncommitted changes.
+        """
         result = self.mainwidget.messagebox.query3("Pending Changes",
             "There are uncommitted changes. Commit them?")
         if result == answerYes:
@@ -279,7 +322,8 @@ class PEditor(object):
         return result
     
     def querysave(self):
-        """ ask user whether to save unsaved files """
+        """Ask user whether to save unsaved files.
+        """
         result = self.mainwidget.messagebox.query3("Unsaved Data",
             "There is unsaved data. Save data?")
         if result == answerYes:
@@ -290,9 +334,9 @@ class PEditor(object):
         return ( ((not self.pending) or (self.querycommit() != answerCancel))
             and ((not self.modified) or (self.querysave() != answerCancel)) )
 
+
 class PFileEditor(PEditor):
-    """
-    An editor class with richer file management capabilities.
+    """Editor class with richer file management capabilities.
     
     Constructor can be passed a file path (typical use expected to be as a
     helper class for a subwindow for file editing), or if not and this is
@@ -305,25 +349,26 @@ class PFileEditor(PEditor):
     captionformat = "%s %s [%s]"
     getfilename = True
     
-    def __init__(self, mainwidget=None, filename="", control=None, data=None):
-        PEditor.__init__(self, mainwidget, control, data)
-        
-        # connect more main widget actions
-        if self.mainwidget is not None:
-            self.mainwidget.connectaction(ACTION_FILENEW, self.newfile)
-            self.mainwidget.connectaction(ACTION_FILEOPEN, self.openfile)
-            self.mainwidget.connectaction(ACTION_FILESAVE, self.savefile)
-            self.mainwidget.connectaction(ACTION_FILESAVEAS, self.savefileas)
+    def __init__(self, mainwidget=None, filename="", control=None):
+        PEditor.__init__(self, mainwidget, control)
         
         # open file
         self.filename = filename
-        if (self.filename == "") and self.getfilename:
-            self.openfile()
-        elif self.filename != "":
+        if self.filename:
             self._dofileopen()
+        elif self.getfilename:
+            self.openfile()
         else:
             self.newfile()
     
+    def _connect_mainwidget(self):
+        super(PFileEditor, self)._connect_mainwidget()
+        # connect more main widget actions
+        self.mainwidget.connectaction(ACTION_FILENEW, self.newfile)
+        self.mainwidget.connectaction(ACTION_FILEOPEN, self.openfile)
+        self.mainwidget.connectaction(ACTION_FILESAVE, self.savefile)
+        self.mainwidget.connectaction(ACTION_FILESAVEAS, self.savefileas)
+    
     def _getname(self):
         return self.filename or "<New File>"
     
@@ -332,40 +377,47 @@ class PFileEditor(PEditor):
         self.switchname()
     
     def newfile(self):
-        """ Create new unnamed file. """
-        self.filename = ""
-        self._dofilenew()
+        """Create new unnamed file.
+        """
+        if self.canclose():
+            self.filename = ""
+            self._dofilenew()
     
     def _dofileopen(self):
-        if self.filename != "":
+        if self.filename:
             self.load()
             self.switchname()
     
     def openfile(self):
-        """ Select file name and open file. """
-        fname = self.mainwidget.getfiledialog()
-        if len(fname) > 0:
-            self.filename = str(fname)
-            self._dofileopen()
+        """Select file name and open file.
+        """
+        if self.canclose():
+            fname = self.mainwidget.getfiledialog()
+            if len(fname) > 0:
+                self.filename = str(fname)
+                self._dofileopen()
     
     def _dofilesave(self):
-        if self.filename != "":
+        if self.filename:
             self.save()
             self.switchname()
     
     def savefile(self):
-        """ Save file to current file name, or select one if there is none. """
-        if self.filename == "":
+        """Save file to current file name, or select one if there is none.
+        """
+        if not self.filename:
             self.savefileas()
         else:
             self._dofilesave()
     
     def savefileas(self):
-        """ Select new file name and save file. """
+        """Select new file name and save file.
+        """
         startpath, startfilter = os.path.splitext(self.filename)
         startpath = os.path.dirname(startpath)
         startfilter = "*" + startfilter
-        fname = self.mainwidget.getfiledialog(startpath, startfilter, ACTION_FILESAVEAS)
+        fname = self.mainwidget.getfiledialog(startpath, startfilter,
+            ACTION_FILESAVEAS)
         if len(fname) > 0:
             self.filename = str(fname)
             self._dofilesave()
