@@ -2,7 +2,7 @@
 """
 Module GTKAPP -- Python GTK Application Objects
 Sub-Package GUI.TOOLKITS.GTK of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -21,6 +21,7 @@ from plib.gui._base import app
 
 from _gtkcommon import _PGtkCommunicator, _gtksignals
 
+
 class PGtkAboutDialog(app.PAboutDialogBase, gtk.AboutDialog):
     
     attrmap = {
@@ -42,10 +43,11 @@ class PGtkAboutDialog(app.PAboutDialogBase, gtk.AboutDialog):
     
     def display(self):
         self.run()
+        self.hide()
+
 
 class _PGtkMainMixin(gtk.Window, _PGtkCommunicator):
-    """
-    Mixin class for GTK top windows and main windows.
+    """Mixin class for GTK top windows and main windows.
     """
     
     aboutdialogclass = PGtkAboutDialog
@@ -99,6 +101,9 @@ class _PGtkMainMixin(gtk.Window, _PGtkCommunicator):
     def _hide_window(self):
         gtk.Window.hide(self)
     
+    def set_iconfile(self, iconfile):
+        self.set_icon_from_file(iconfile)
+    
     def set_caption(self, caption):
         self.set_title(caption)
     
@@ -122,23 +127,25 @@ class _PGtkMainMixin(gtk.Window, _PGtkCommunicator):
     
     def exit(self):
         # 'automagic' method to fake SIGNAL_QUERYCLOSE when window closes in
-        # response to a program signal (wtf isn't this a method in the GTK API?)
+        # response to a program signal
+        # (wtf isn't this a method in the GTK API?)
         if not self.emit("delete_event", gtk.gdk.Event(gtk.gdk.DELETE)):
             # Send the closing signal
             self.do_notify(SIGNAL_CLOSING)
             self.destroy()
     
     def delete_event(self):
-        # 'automagic' method for SIGNAL_QUERYCLOSE (btw, while we're venting, wtf
-        # do we have to return False to indicate that the window *can* close?)
+        # 'automagic' method for SIGNAL_QUERYCLOSE
         return not self.acceptclose()
     
     def destroy_event(self):
-        # If we're the main window, this is the 'automagic' method for SIGNAL_BEFOREQUIT
+        # If we're the main window, this is the 'automagic' method for
+        # SIGNAL_BEFOREQUIT
         if self.app.mainwin == self:
             self.app.before_quit()
             gtk.main_quit()
 
+
 class PGtkTopWindow(_PGtkMainMixin, app.PTopWindowBase):
     
     def __init__(self, parent, cls=None):
@@ -154,15 +161,13 @@ class PGtkTopWindow(_PGtkMainMixin, app.PTopWindowBase):
         app.PTopWindowBase.show_init(self)
         _PGtkMainMixin.show_init(self)
 
+
 class PGtkApplication(app.PApplicationBase, _PGtkCommunicator):
-    """
-    Customized GTK application class.
+    """Customized GTK application class.
     """
     
-    mainwidgetclass = PGtkTopWindow
-    
-    def __init__(self, arglist=[], cls=None):
-        app.PApplicationBase.__init__(self, arglist, cls)
+    def __init__(self, arglist=[], cls=None, use_mainwindow=False):
+        app.PApplicationBase.__init__(self, arglist, cls, use_mainwindow)
         self.mainwin = self.createMainWidget()
     
     def connect(self, signal, target):
