@@ -2,7 +2,7 @@
 """
 Module GTKCOMMON -- Python GTK Common Objects
 Sub-Package GUI.TOOLKITS.GTK of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -29,6 +29,12 @@ _gtkstockids = {
     ACTION_FILESAVEAS: gtk.STOCK_SAVE_AS,
     ACTION_FILECLOSE: gtk.STOCK_CLOSE,
     ACTION_EDIT: gtk.STOCK_EDIT,
+    ACTION_EDITUNDO: gtk.STOCK_UNDO,
+    ACTION_EDITREDO: gtk.STOCK_REDO,
+    ACTION_EDITCUT: gtk.STOCK_CUT,
+    ACTION_EDITCOPY: gtk.STOCK_COPY,
+    ACTION_EDITPASTE: gtk.STOCK_PASTE,
+    ACTION_EDITDELETE: gtk.STOCK_DELETE,
     ACTION_REFRESH: gtk.STOCK_REFRESH,
     ACTION_ADD: gtk.STOCK_ADD,
     ACTION_REMOVE: gtk.STOCK_REMOVE,
@@ -39,6 +45,10 @@ _gtkstockids = {
     ACTION_ABOUT: gtk.STOCK_ABOUT,
     ACTION_EXIT: gtk.STOCK_QUIT }
 
+# Need this because earlier GTK versions lack this stock id
+if hasattr(gtk, 'STOCK_SELECT_ALL'):
+    _gtkstockids[ACTION_EDITSELECTALL] = gtk.STOCK_SELECT_ALL
+
 _gtkicons = {
     MBOX_INFO: gtk.MESSAGE_INFO,
     MBOX_WARN: gtk.MESSAGE_WARNING,
@@ -58,9 +68,10 @@ _gtksignals = {
     SIGNAL_SELECTED: "selected",
     SIGNAL_FOCUS_IN: "focus_in_event",
     SIGNAL_FOCUS_OUT: "focus_out_event",
-    SIGNAL_LISTVIEWCHANGED: "listview_changed",
+    SIGNAL_LISTSELECTED: "listview_changed",
     SIGNAL_TABLECHANGED: "table_changed",
     SIGNAL_TEXTCHANGED: "changed",
+    SIGNAL_TEXTSTATECHANGED: "state_changed",
     SIGNAL_EDITCHANGED: "changed",
     SIGNAL_ENTER: "enter_pressed",
     SIGNAL_TABCHANGED: "tab_changed",
@@ -78,12 +89,17 @@ _gtksignals = {
 # 'Wrapper' functions for certain signals to discard object parameter
 # (since the way we're set up it will always be the same as self anyway)
 
-_gtkwrappers = [SIGNAL_ACTIVATED, SIGNAL_CLICKED, SIGNAL_TOGGLED, SIGNAL_SELECTED,
-    SIGNAL_LISTVIEWCHANGED, SIGNAL_TABLECHANGED,
-    SIGNAL_TEXTCHANGED, SIGNAL_EDITCHANGED, SIGNAL_ENTER,
+_gtkwrappers = [
+    SIGNAL_ACTIVATED, SIGNAL_CLICKED, SIGNAL_TOGGLED,
+    SIGNAL_SELECTED,
+    SIGNAL_LISTSELECTED, SIGNAL_TABLECHANGED,
+    SIGNAL_TEXTCHANGED, SIGNAL_TEXTSTATECHANGED,
+    SIGNAL_EDITCHANGED, SIGNAL_ENTER,
     SIGNAL_TABCHANGED]
 _gtkplain = [SIGNAL_FOCUS_IN, SIGNAL_FOCUS_OUT,
-    SIGNAL_QUERYCLOSE, SIGNAL_CLOSING, SIGNAL_HIDDEN, SIGNAL_BEFOREQUIT]
+    SIGNAL_QUERYCLOSE, SIGNAL_CLOSING,
+    SIGNAL_HIDDEN, SIGNAL_BEFOREQUIT]
+
 
 def _gtk_wrapper(target, plain=False):
     if plain:
@@ -94,9 +110,9 @@ def _gtk_wrapper(target, plain=False):
             target(*args)
     return wrapper
 
+
 class _PGtkCommunicator(object):
-    """
-    A mixin class to abstract signal functionality in GTK.
+    """Mixin class to abstract signal functionality in GTK.
     """
     
     def setup_notify(self, signal, target, wrap=True):
@@ -113,13 +129,22 @@ class _PGtkCommunicator(object):
     def do_notify(self, signal, *args):
         self.emit(_gtksignals[signal], *args)
 
+
+_gtk_font_weights = [pango.WEIGHT_NORMAL, pango.WEIGHT_BOLD]
+_gtk_font_styles = [pango.STYLE_NORMAL, pango.STYLE_ITALIC]
+
+
 class _PGtkWidgetBase(object):
-    """ Mixin class to provide basic GTK widget methods. """
+    """Mixin class to provide basic GTK widget methods.
+    """
+    
+    fn_enable_get = 'get_sensitive'
+    fn_enable_set = 'set_sensitive'
     
     def _widget(self):
         return self
     
-    def update(self):
+    def update_widget(self):
         self._widget().queue_draw()
     
     def preferred_width(self):
@@ -129,26 +154,22 @@ class _PGtkWidgetBase(object):
         return self._widget().get_size_request()[1]
     
     def set_width(self, width):
-        height = self.get_size_request()[1]
-        self.set_size_request(width, height)
+        height = self._widget().get_size_request()[1]
+        self._widget().set_size_request(width, height)
     
     def set_height(self, height):
-        width = self.get_size_request()[0]
-        self.set_size_request(width, height)
+        width = self._widget().get_size_request()[0]
+        self._widget().set_size_request(width, height)
     
     def set_position(self, left, top):
         if not (None in (left, top)):
-            self._parent._move_widget(self, left, top)
+            self._parent._move_widget(self._widget(), left, top)
     
     def set_foreground_color(self, color):
         pass # FIXME
     
-    def _font_info(self, font_name, font_size=None):
-        #if font_name in _gtkfontfamilies:
-        #    font_name = _gtkfontfamilies[font_name]
-        if font_size is None:
-            font_size = default_font_size
-        return font_name, font_size
+    def set_background_color(self, color):
+        pass # FIXME
     
     def _font_widget(self):
         # Some Gtk widgets, like buttons, actually are containers and a child
@@ -156,38 +177,56 @@ class _PGtkWidgetBase(object):
         # factored out
         return self
     
+    # Chained method calls make it hard to meet PEP 8 80 chars per line...
+    
     def get_font_name(self):
         return self._font_widget().get_pango_context().get_font_description().get_family()
     
     def get_font_size(self):
-        return self._font_widget().get_pango_context().get_font_description().get_size()
+        return (self._font_widget().get_pango_context().get_font_description().get_size() / pango.SCALE)
     
-    def set_font(self, font_name, font_size=None):
-        font_name, font_size = self._font_info(font_name, font_size)
-        self._font_widget().modify_font(pango.FontDescription("%s %s" % (font_name, font_size)))
+    def get_font_bold(self):
+        return (self._font_widget().get_pango_context().get_font_description().get_weight()
+            == _gtk_font_weights[1])
     
-    def _get_enabled(self):
-        return self.get_sensitive()
+    def get_font_italic(self):
+        return (self._font_widget().get_pango_context().get_font_description().get_style()
+            == _gtk_font_styles[1])
     
-    def _set_enabled(self, value):
-        self.set_sensitive(value)
+    def _gtk_font_desc(self, font_name, font_size, bold, italic):
+        styles = []
+        if bold:
+            styles.append("bold")
+        if italic:
+            styles.append("italic")
+        return pango.FontDescription("%s %s %s" %
+            (font_name, " ".join(styles), font_size))
     
-    enabled = property(_get_enabled, _set_enabled)
+    def _set_font_object(self, font_name, font_size, bold, italic):
+        self._font_widget().modify_font(self._gtk_font_desc(
+            font_name, font_size, bold, italic))
     
     def set_focus(self):
-        self.grab_focus()
+        self._widget().grab_focus()
+
 
 class _PGtkWidget_(_PGtkCommunicator, _PGtkWidgetBase):
-    # Intermediate class so _PGtkClientWidget doesn't inherit the overridden __init__ method
+    # Intermediate class so _PGtkClientWidget doesn't inherit the
+    # overridden __init__ method
     pass
 
+
 class _PGtkWidget(_PGtkWidget_):
-    """ Mixin class for GTK widgets that can send/receive signals. """
+    """Mixin class for GTK widgets that can send/receive signals.
+    """
     
     def __init__(self, parent, align=ALIGN_LEFT):
         self._parent = parent
         self._align = align
         parent._add_widget(self)
 
+
 class _PGtkClientWidget(_PGtkWidget_):
-    """ Mixin class for GTK client widgets (i.e., can be used as client of main window). """
+    """Mixin class for GTK main window client widgets.
+    """
+    pass
