@@ -2,7 +2,7 @@
 """
 Module GTKSTATUSBAR -- Python Gtk Status Bar Objects
 Sub-Package GUI.TOOLKITS.GTK of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -16,9 +16,11 @@ import gtk
 
 from plib.gui._base import statusbar
 
+from _gtkcommon import _PGtkWidgetBase
 #from _gtklabel import PGtkTextLabel
 
-class PGtkStatusBar(gtk.Statusbar, statusbar.PStatusBarBase):
+
+class PGtkStatusBar(gtk.Statusbar, _PGtkWidgetBase, statusbar.PStatusBarBase):
     
     #textareaclass = PWxTextLabel # FIXME: not needed for gtk?
     
