@@ -2,7 +2,7 @@
 """
 Module GTKTABWIDGET -- Python Gtk Tab Widget
 Sub-Package GUI.TOOLKITS.GTK of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -20,7 +20,9 @@ from plib.gui._widgets import tabwidget
 
 from _gtkcommon import _PGtkClientWidget, _gtksignals
 
-class PGtkTabWidget(gtk.Notebook, _PGtkClientWidget, tabwidget.PTabWidgetBase):
+
+class PGtkTabWidget(gtk.Notebook, _PGtkClientWidget,
+        tabwidget.PTabWidgetBase):
     
     # Define tab changed signal using 'automagic' class field
     __gsignals__ = { _gtksignals[SIGNAL_TABCHANGED]:
@@ -33,13 +35,14 @@ class PGtkTabWidget(gtk.Notebook, _PGtkClientWidget, tabwidget.PTabWidgetBase):
         gtk.Notebook.__init__(self)
         tabwidget.PTabWidgetBase.__init__(self, parent, tabs, target)
         
-        # The tab changed signal doesn't include a usable pointer to the new widget,
-        # so gateway it
-        self._initialized = False # this prevents firing an extra event on startup
+        # The tab changed signal doesn't include a usable pointer to the
+        # new widget, so gateway it
+        self._initialized = False  # prevent firing an extra event on startup
         self.connect("switch_page", self.tab_switched)
     
     def tab_switched(self, obj, page, index):
-        """ New tab was selected. """
+        """New tab was selected.
+        """
         if self._initialized:
             self.do_notify(SIGNAL_TABCHANGED, self._items[index])
         else:
