@@ -2,7 +2,7 @@
 """
 Module KDEAPP -- Python KDE Application Objects
 Sub-Package GUI.TOOLKITS.KDE of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -21,29 +21,41 @@ from plib.gui._base import app
 
 from _kdecommon import _PKDECommunicator
 
+# We'll need this for the about dialog hackery below
+
+_kdeversion_items = tuple(map(
+    lambda name: getattr(kdecore.KDE, 'version%s' % name)(),
+    ('Major', 'Minor', 'Release')))
+
+
+def _kdeabout(name, version):
+    return kdecore.KAboutData(name, name, version)
+
+
 def _kdeparse(aboutdata):
-    data = kdecore.KAboutData(aboutdata['name'], aboutdata['name'], aboutdata['version'])
+    data = _kdeabout(aboutdata['name'], aboutdata['version'])
     data.setCopyrightStatement(aboutdata['copyright'])
     data.setLicenseText(aboutdata['license'])
     data.setShortDescription(aboutdata['description'])
     for dev in aboutdata['developers']:
         data.addAuthor(dev)
     data.setHomepage(aboutdata['website'])
-    # FIXME: wtf doesn't this work? Alternately, why doesn't setting the program
-    # icon elsewhere cause it to display here?
-    #data.setProgramLogo(qt.QPixmap(aboutdata['icon']))
     return data
 
+
+def _kdelogo(icon, kaboutdata):
+    qimg = qt.QImage(icon)
+    qimg_scaled = qimg.scale(qimg.width() * 2, qimg.height() * 2)
+    kaboutdata.setProgramLogo(qimg_scaled)
+
+
 class _PKDEMainMixin(kdeui.KMainWindow, _PKDECommunicator):
-    """
-    Mixin class for KDE top windows and main windows.
+    """Mixin class for KDE top windows and main windows.
     """
     
-    _closed = False # default for guard to trap close from system menu, see below
-    
-    def _init_icon(self):
-        if (self.aboutdata is not None) and ('icon' in self.aboutdata):
-            self.setIcon(qt.QPixmap(self.aboutdata['icon']))
+    _aboutdlg = None
+    _aboutkde = None
+    _closed = False  # guard to trap close from system menu, see below
     
     def _get_w(self):
         return self.width()
@@ -55,6 +67,9 @@ class _PKDEMainMixin(kdeui.KMainWindow, _PKDECommunicator):
     def _hide_window(self):
         kdeui.KMainWindow.hide(self)
     
+    def set_iconfile(self, iconfile):
+        self.setIcon(qt.QPixmap(iconfile))
+    
     def set_caption(self, caption):
         self.setPlainCaption(caption)
     
@@ -66,7 +81,9 @@ class _PKDEMainMixin(kdeui.KMainWindow, _PKDECommunicator):
                 self._showMax = True
         else:
             desktop = self.app.desktop()
-            self.resize(desktop.width() - self.sizeoffset, desktop.height() - self.sizeoffset)
+            self.resize(
+                desktop.width() - self.sizeoffset,
+                desktop.height() - self.sizeoffset)
     
     def sizetoclient(self, clientwidth, clientheight):
         self.resize(clientwidth, clientheight)
@@ -85,17 +102,23 @@ class _PKDEMainMixin(kdeui.KMainWindow, _PKDECommunicator):
             kdeui.KMainWindow.show(self)
     
     def about(self):
-        if self.aboutdata is not None:
+        # FIXME: segfaults when creating the dialog on some KDE 3.5 versions
+        # when used with socket notifier
+        if (self.aboutdata is not None):
             kaboutdata = _kdeparse(self.aboutdata)
-            dlg = kdeui.KAboutApplication(kaboutdata)
-            dlg # FIXME: wtf do we need this to prevent a segfault in the next line?
-            dlg.exec_loop()
-            #kdeui.KAboutApplication(kaboutdata).exec_loop()
+            if 'icon' in self.aboutdata:
+                _kdelogo(self.aboutdata['icon'], kaboutdata)
+            kdeui.KAboutApplication(kaboutdata).exec_loop()
+    
+    def about_toolkit(self):
+        # FIXME: segfaults when creating the dialog on some KDE 3.5 versions
+        # when used with socket notifier
+        kdeui.KAboutKDE().exec_loop()
     
     def exit(self):
-        # Guard traps a close other than from this method, so we don't throw an
-        # exception if this method gets called after we close but before shutdown
-        # (?? Why isn't this necessary in Qt?)
+        # Guard traps a close other than from this method, so we don't
+        # throw an exception if this method gets called after we close
+        # but before shutdown (?? Why isn't this necessary in Qt?)
         if not self._closed:
             self.close()
     
@@ -103,6 +126,9 @@ class _PKDEMainMixin(kdeui.KMainWindow, _PKDECommunicator):
         # 'automagic' code for SIGNAL_QUERYCLOSE
         if self.acceptclose():
             self._emit_event(SIGNAL_CLOSING)
+            for dlg in (self._aboutdlg, self._aboutkde):
+                if dlg is not None:
+                    dlg.done(0)
             self._closed = True # set guard used above
             event.accept()
         else:
@@ -111,45 +137,54 @@ class _PKDEMainMixin(kdeui.KMainWindow, _PKDECommunicator):
     def hideEvent(self, event):
         self._emit_event(SIGNAL_HIDDEN)
 
+
 class PKDETopWindow(_PKDEMainMixin, app.PTopWindowBase):
+    """Customized KDE top window class.
+    """
     
     def __init__(self, parent, cls=None):
         _PKDEMainMixin.__init__(self)
         app.PTopWindowBase.__init__(self, parent, cls)
         self.setCentralWidget(self.clientwidget)
-        # This has to be at the end so app.PTopWindowBase.__init__ has a chance
-        # to read the about data from the client widget, if applicable
-        self._init_icon()
     
     def show_init(self):
         app.PTopWindowBase.show_init(self)
         _PKDEMainMixin.show_init(self)
 
-class PKDEApplication(kdecore.KApplication, app.PApplicationBase, _PKDECommunicator):
-    """
-    Customized KDE application class.
+
+class PKDEApplication(kdecore.KApplication, app.PApplicationBase,
+        _PKDECommunicator):
+    """Customized KDE application class.
     """
     
-    mainwidgetclass = PKDETopWindow
+    _in_local_loop = False
     
-    def __init__(self, arglist=[], cls=None):
+    def __init__(self, arglist=[], cls=None, use_mainwindow=False):
         if cls is None:
             klass = self.mainwidgetclass
         else:
             klass = cls
+        
         if hasattr(klass, 'aboutdata') and (klass.aboutdata is not None):
             aboutdata = klass.aboutdata
-        elif hasattr(klass, 'clientwidgetclass') and hasattr(klass.clientwidgetclass, 'aboutdata'):
+        
+        elif hasattr(klass, 'clientwidgetclass') and \
+                hasattr(klass.clientwidgetclass, 'aboutdata'):
+            
             aboutdata = klass.clientwidgetclass.aboutdata
+        
         else:
             aboutdata = None
-        if aboutdata is not None:
+        
+        if aboutdata:
             kaboutdata = _kdeparse(aboutdata)
         else:
-            kaboutdata = kdecore.KAboutData('unnamed', 'unnamed', '0.0')
-        kdecore.KCmdLineArgs.init(kaboutdata) # all the above because KDE requires this incantation first
+            kaboutdata = _kdeabout("Unnamed", "0.0")
+        
+        # All the above because KDE requires this incantation first
+        kdecore.KCmdLineArgs.init(kaboutdata)
         kdecore.KApplication.__init__(self)
-        app.PApplicationBase.__init__(self, arglist, cls)
+        app.PApplicationBase.__init__(self, arglist, cls, use_mainwindow)
         self.mainwin = self.createMainWidget()
         self.setMainWidget(self.mainwin)
         
@@ -161,3 +196,19 @@ class PKDEApplication(kdecore.KApplication, app.PApplicationBase, _PKDECommunica
     
     def process_events(self):
         self.processEvents()
+    
+    # For use when multiplexing with other event types (e.g.,
+    # in a NotifierClient
+    
+    # FIXME: This only seems to work for KDE 3 when enterLoop() and
+    # exitLoop() are called directy from NotifierClient ???
+    
+    #def enter_yield(self):
+    #    if not self._in_local_loop:
+    #        self.eventLoop().enterLoop()
+    #        self._in_local_loop = True
+    
+    #def exit_yield(self):
+    #    if self._in_local_loop:
+    #        self._in_local_loop = False
+    #        self.eventLoop().exitLoop()
