@@ -2,7 +2,7 @@
 """
 Module WXAPP -- Python wxWidgets Application Objects
 Sub-Package GUI.TOOLKITS.WX of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -19,6 +19,12 @@ from plib.gui._base import app
 
 from _wxcommon import _PWxCommunicator
 
+
+def _wx_icon_from_file(filename):
+    # Kludgy but gets the job done on all platforms
+    return wx.IconFromBitmap(wx.BitmapFromImage(wx.Image(filename)))
+
+
 class PWxAboutDialog(app.PAboutDialogBase, wx.AboutDialogInfo):
     
     attrmap = {
@@ -36,23 +42,14 @@ class PWxAboutDialog(app.PAboutDialogBase, wx.AboutDialogInfo):
         app.PAboutDialogBase.__init__(self, parent)
     
     def SetIconFromFile(self, filename):
-        if sys.platform == 'darwin':
-            icon = wx.IconFromBitmap(wx.BitmapFromImage(wx.Image(filename)))
-        else:
-            if sys.platform == 'win32':
-                # TODO: load win32 icon from resource instead of bare file
-                icontype = wx.BITMAP_TYPE_ICO
-            else:
-                icontype = wx.BITMAP_TYPE_XPM
-            icon = wx.Icon(filename, icontype)
-        self.SetIcon(icon)
+        self.SetIcon(_wx_icon_from_file(filename))
     
     def display(self):
         wx.AboutBox(self)
 
+
 class _PWxMainMixin(wx.Frame, _PWxCommunicator):
-    """
-    Mixin class for wxWidgets top windows and main windows.
+    """Mixin class for wxWidgets top windows and main windows.
     """
     
     aboutdialogclass = PWxAboutDialog
@@ -67,6 +64,9 @@ class _PWxMainMixin(wx.Frame, _PWxCommunicator):
     def _hide_window(self):
         wx.Frame.Hide(self)
     
+    def set_iconfile(self, iconfile):
+        self.SetIcon(_wx_icon_from_file(iconfile))
+    
     def set_caption(self, caption):
         self.SetTitle(caption)
     
@@ -101,6 +101,7 @@ class _PWxMainMixin(wx.Frame, _PWxCommunicator):
             self.ProcessPendingEvents()
             self.Destroy()
 
+
 class PWxTopWindow(_PWxMainMixin, app.PTopWindowBase):
     
     def __init__(self, parent, cls=None):
@@ -114,19 +115,17 @@ class PWxTopWindow(_PWxMainMixin, app.PTopWindowBase):
         app.PTopWindowBase.show_init(self)
         _PWxMainMixin.show_init(self)
 
+
 class PWxApplication(wx.App, app.PApplicationBase, _PWxCommunicator):
-    """
-    Customized wxWidgets application class.
+    """Customized wxWidgets application class.
     """
     
-    mainwidgetclass = PWxTopWindow
-    
-    def __init__(self, arglist=[], cls=None):
-        app.PApplicationBase.__init__(self, arglist, cls)
+    def __init__(self, arglist=[], cls=None, use_mainwindow=False):
+        app.PApplicationBase.__init__(self, arglist, cls, use_mainwindow)
         wx.App.__init__(self, arglist)
     
     def OnInit(self):
-        """ wxWidgets wants you to initialize subwidgets here """
+        # wxWidgets wants you to initialize subwidgets here
         
         self.mainwin = self.createMainWidget()
         self.SetTopWindow(self.mainwin)
@@ -142,6 +141,6 @@ class PWxApplication(wx.App, app.PApplicationBase, _PWxCommunicator):
         self.MainLoop()
     
     def process_events(self):
-        #self.ProcessPendingEvents()
-        self.Yield()
+        self.ProcessPendingEvents()
+        #self.Yield()
         #wx.SafeYield()
