@@ -2,7 +2,7 @@
 """
 Module WXCOMMON -- Python wxWidgets Common Objects
 Sub-Package GUI.TOOLKITS.WX of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -10,6 +10,8 @@ See the LICENSE and README files for more information
 This module contains the wxWidgets GUI common objects.
 """
 
+import string
+
 import wx
 import wx.grid
 from wx.lib.evtmgr import eventManager
@@ -17,6 +19,12 @@ from wx.lib.newevent import NewEvent
 
 from plib.gui.defs import *
 
+# Style constant for edit controls that are scrolling
+
+_wx_ec_scroll_style = wx.VSCROLL | wx.HSCROLL | wx.TE_DONTWRAP
+
+# FIXME: These are only good for buttons and menus, and the toolbar ones
+# in wx.ArtProvider don't match all the ids! WTF!@#$%^&*()
 _wxstockids = {
     ACTION_FILENEW: wx.ID_NEW,
     ACTION_FILEOPEN: wx.ID_OPEN,
@@ -24,6 +32,13 @@ _wxstockids = {
     ACTION_FILESAVEAS: wx.ID_SAVEAS,
     ACTION_FILECLOSE: wx.ID_CLOSE,
     ACTION_EDIT: wx.ID_EDIT,
+    ACTION_EDITUNDO: wx.ID_UNDO,
+    ACTION_EDITREDO: wx.ID_REDO,
+    ACTION_EDITCUT: wx.ID_CUT,
+    ACTION_EDITCOPY: wx.ID_COPY,
+    ACTION_EDITPASTE: wx.ID_PASTE,
+    ACTION_EDITDELETE: wx.ID_DELETE,
+    ACTION_EDITSELECTALL: wx.ID_SELECTALL,
     ACTION_REFRESH: wx.ID_REFRESH,
     ACTION_ADD: wx.ID_ADD,
     ACTION_REMOVE: wx.ID_REMOVE,
@@ -35,9 +50,9 @@ _wxstockids = {
     ACTION_EXIT: wx.ID_EXIT }
 
 _wxalignmap = {
-    ALIGN_LEFT: None,
-    ALIGN_CENTER: None,
-    ALIGN_RIGHT: None }
+    ALIGN_LEFT: wx.LIST_FORMAT_LEFT,
+    ALIGN_CENTER: wx.LIST_FORMAT_CENTRE,
+    ALIGN_RIGHT: wx.LIST_FORMAT_RIGHT }
 
 _wxicons = {
     MBOX_INFO: wx.ICON_INFORMATION,
@@ -46,13 +61,16 @@ _wxicons = {
     MBOX_QUERY: wx.ICON_QUESTION }
 
 _wxfontfamilies = {
-    wx.FONTFAMILY_ROMAN: ["Courier New", "Times New Roman"],
-    wx.FONTFAMILY_SWISS: ["Arial", "Verdana"] }
+    wx.FONTFAMILY_ROMAN: ["Times New Roman"],
+    wx.FONTFAMILY_SWISS: ["Arial", "Verdana"],
+    wx.FONTFAMILY_MODERN: ["Courier New"] }
 
-# Define our own custom event for SIGNAL_CLOSING
+# Define our own custom events
 ClosingEvent, EVT_CLOSING = NewEvent()
+TextStateEvent, EVT_TEXT_STATE_CHANGED = NewEvent()
 
 _wxcustommap = {
+    SIGNAL_TEXTSTATECHANGED: TextStateEvent,
     SIGNAL_CLOSING: ClosingEvent }
 
 _wxeventmap = {
@@ -62,9 +80,10 @@ _wxeventmap = {
     SIGNAL_SELECTED: wx.EVT_COMBOBOX,
     SIGNAL_FOCUS_IN: wx.EVT_SET_FOCUS,
     SIGNAL_FOCUS_OUT: wx.EVT_KILL_FOCUS,
-    SIGNAL_LISTVIEWCHANGED: wx.EVT_TREE_SEL_CHANGED,
+    SIGNAL_LISTSELECTED: wx.EVT_TREE_SEL_CHANGED,
     SIGNAL_TABLECHANGED: wx.grid.EVT_GRID_CELL_CHANGE,
     SIGNAL_TEXTCHANGED: wx.EVT_TEXT,
+    SIGNAL_TEXTSTATECHANGED: EVT_TEXT_STATE_CHANGED,
     SIGNAL_EDITCHANGED: wx.EVT_TEXT,
     SIGNAL_ENTER: wx.EVT_TEXT_ENTER,
     SIGNAL_TABCHANGED: wx.EVT_NOTEBOOK_PAGE_CHANGED,
@@ -73,6 +92,7 @@ _wxeventmap = {
     SIGNAL_QUERYCLOSE: wx.EVT_CLOSE,
     SIGNAL_BEFOREQUIT: wx.EVT_WINDOW_DESTROY }
 
+
 # 'Wrapper' functions for certain events to repackage parameters
 
 def wx_plain_wrapper(self, target):
@@ -80,11 +100,13 @@ def wx_plain_wrapper(self, target):
         target()
     return wrapper
 
+
 def wx_selected_wrapper(self, target):
     def wrapper(event):
         target(self.GetSelection())
     return wrapper
 
+
 def wx_listviewchanged_wrapper(self, target):
     def wrapper(event):
         item = self.GetItemPyData(event.GetItem())
@@ -93,16 +115,19 @@ def wx_listviewchanged_wrapper(self, target):
             target(item)
     return wrapper
 
+
 def wx_tablechanged_wrapper(self, target):
     def wrapper(event):
         target(event.GetRow(), event.GetCol())
     return wrapper
 
+
 def wx_tabchanged_wrapper(self, target):
     def wrapper(event):
         target(self._items[event.GetSelection()])
     return wrapper
 
+
 _wxwrappermap = {
     SIGNAL_ACTIVATED: wx_plain_wrapper,
     SIGNAL_CLICKED: wx_plain_wrapper,
@@ -110,7 +135,7 @@ _wxwrappermap = {
     SIGNAL_SELECTED: wx_selected_wrapper,
     SIGNAL_FOCUS_IN: wx_plain_wrapper,
     SIGNAL_FOCUS_OUT: wx_plain_wrapper,
-    SIGNAL_LISTVIEWCHANGED: wx_listviewchanged_wrapper,
+    SIGNAL_LISTSELECTED: wx_listviewchanged_wrapper,
     SIGNAL_TABLECHANGED: wx_tablechanged_wrapper,
     SIGNAL_TEXTCHANGED: wx_plain_wrapper,
     SIGNAL_EDITCHANGED: wx_plain_wrapper,
@@ -119,9 +144,9 @@ _wxwrappermap = {
     SIGNAL_CLOSING: wx_plain_wrapper,
     SIGNAL_HIDDEN: wx_plain_wrapper }
 
+
 class _PWxCommunicator(object):
-    """
-    A mixin class to abstract notification functionality in wxWidgets.
+    """Mixin class to abstract notification functionality in wxWidgets.
     """
     
     def setup_notify(self, signal, target, wrap=True):
@@ -144,16 +169,30 @@ class _PWxCommunicator(object):
             event.SetEventType(_wxeventmap[signal])
             if hasattr(self, 'id'):
                 event.SetEventID(self.id)
+            elif hasattr(self, '_id'):
+                event.SetEventID(self._id)
             elif hasattr(self, 'GetId'):
                 event.SetEventID(self.GetId())
         if event is not None:
             event.SetEventObject(self)
             self.AddPendingEvent(event)
 
+
+_wx_font_weights = [wx.FONTWEIGHT_NORMAL, wx.FONTWEIGHT_BOLD]
+_wx_font_styles = [wx.FONTSTYLE_NORMAL, wx.FONTSTYLE_ITALIC]
+
+
 class _PWxWidgetBase(object):
-    """ Mixin class to provide basic wx widget methods. """
+    """Mixin class to provide basic wx widget methods.
+    """
     
-    def update(self):
+    fn_enable_get = 'IsEnabled'
+    fn_enable_set = 'Enable'
+    
+    _depth_test_str = string.ascii_letters + string.digits
+    _depth_scale_factor = 1.3
+    
+    def update_widget(self):
         self.Refresh()
     
     def preferred_width(self):
@@ -179,43 +218,60 @@ class _PWxWidgetBase(object):
     def set_foreground_color(self, color):
         self.SetForegroundColour(color)
     
+    def set_background_color(self, color):
+        self.SetBackgroundColour(color)
+    
     def get_font_name(self):
-        font = self.GetFont()
-        return font.GetFaceName()
+        return self.GetFont().GetFaceName()
     
     def get_font_size(self):
-        font = self.GetFont()
-        return font.GetPointSize()
+        return self.GetFont().GetPointSize()
     
-    def set_font(self, font_name, font_size=None):
-        if font_size is None:
-            font_size = default_font_size
+    def get_font_bold(self):
+        return (self.GetFont().GetWeight() == _wx_font_weights[1])
+    
+    def get_font_italic(self):
+        return (self.GetFont().GetStyle() == _wx_font_styles[1])
+    
+    def _wx_font_object(self, font_name, font_size, bold, italic):
         font_family = wx.FONTFAMILY_DEFAULT
         for family, names in _wxfontfamilies.iteritems():
             if font_name in names:
                 font_family = family
                 break
-        font_style = wx.FONTSTYLE_NORMAL
-        font_weight = wx.FONTWEIGHT_NORMAL
+        font_style = _wx_font_styles[int(italic)]
+        font_weight = _wx_font_weights[int(bold)]
         font = wx.Font(font_size, font_family, font_style, font_weight)
         font.SetFaceName(font_name)
+        return font
+    
+    def _set_font_object(self, font_name, font_size, bold, italic):
+        font = self._wx_font_object(font_name, font_size, bold, italic)
+        if hasattr(self, '_depth_w'):
+            # Hack to make list view column auto-sizing work correctly
+            # on font change
+            old_extent = self.GetTextExtent(self._depth_test_str)[0]
+            new_extent = self.GetFullTextExtent(self._depth_test_str, font)[0]
+            self._depth_w = int(
+                self._depth_w * self._depth_scale_factor * new_extent /
+                old_extent)
         self.SetFont(font)
-    
-    def _get_enabled(self):
-        return self.IsEnabled()
-    
-    def _set_enabled(self, value):
-        self.Enable(value)
-    
-    enabled = property(_get_enabled, _set_enabled)
+        if hasattr(self, 'SetDefaultCellFont'):
+            self.SetDefaultCellFont(font) # takes care of the table widget
+        if hasattr(self, 'AutoSizeCols'):
+            self.AutoSizeCols() # this takes care of the list view widget
     
     def set_focus(self):
         self.SetFocus()
 
+
 class _PWxWidget(_PWxCommunicator, _PWxWidgetBase):
-    """ Mixin class for wx widgets that can send/receive signals. """
+    """Mixin class for wx widgets that can send/receive signals.
+    """
     pass
 
+
 class _PWxClientWidget(_PWxWidget):
-    """ Mixin class for wx client widgets (i.e., can be used as client of main window). """
+    """Mixin class for wx main window client widgets.
+    """
     pass
