@@ -2,7 +2,7 @@
 """
 Module WXDISPLAY -- Python wxWidgets Text Display Widgets
 Sub-Package GUI.TOOLKITS.WX of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -15,7 +15,8 @@ import wx
 from plib.gui.defs import *
 from plib.gui._widgets import display
 
-from _wxcommon import _PWxWidget
+from _wxcommon import _PWxWidget, _wx_ec_scroll_style
+
 
 class PWxTextDisplay(wx.TextCtrl, _PWxWidget, display.PTextDisplayBase):
     
@@ -25,7 +26,7 @@ class PWxTextDisplay(wx.TextCtrl, _PWxWidget, display.PTextDisplayBase):
         self._expand = True
         self._align = ALIGN_JUST
         if scrolling:
-            self._style = self._style | wx.VSCROLL | wx.HSCROLL | wx.TE_DONTWRAP
+            self._style = self._style | _wx_ec_scroll_style
         wx.TextCtrl.__init__(self, parent, style=self._style)
         self.SetEditable(False)
         display.PTextDisplayBase.__init__(self, text, geometry)
