@@ -2,7 +2,7 @@
 """
 Module HELPERS -- GUI Widget Helper Classes
 Sub-Package GUI.WIDGETS of Package PLIB -- Python GUI Framework
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -25,51 +25,54 @@ container structures supported are:
 
 from plib.stdlib import abstractlist, basesequence, normalize, SortMixin
 
+from plib.gui.common import font_args
+
+
 class _PHelperData(object):
-    """
-    Abstract base class for helper items that update displayed
-    text from data.
-    """
+    # Abstract base class for helper items that update displayed
+    # text from data.
     
     def __init__(self, data=None):
         if data is not None:
             self._update(data)
     
     def _update(self, data):
-        """ Override to populate widget from data. """
+        """Derived classes must implement to populate widget from data.
+        """
         raise NotImplementedError
 
+
 class _PHelperItem(_PHelperData):
-    """
-    Abstract mixin class for helper items. See the PHelper docstring
-    below for more details.
-    """
+    # Abstract mixin class for helper items. See the PHelper docstring
+    # below for more details.
     
     def __init__(self, index, data=None):
         self._i = index
         _PHelperData.__init__(self, data)
 
+
 class _PColsMixin(basesequence):
-    """
-    Mixin class to expose multiple columns of a single row
-    or node in a multi-column widget as a sequence. Designed to
-    be a mixin or helper for a _PHelperItem subclass. Assumes
-    that data is a sequence of strings.
-    """
+    # Mixin class to expose multiple columns of a single row
+    # or node in a multi-column widget as a sequence. Designed to
+    # be a mixin or helper for a _PHelperItem subclass. Assumes
+    # that data is a sequence of strings.
+    
+    _updating = False
     
     def _update(self, data):
+        self._updating = True
         self._len = len(data)
         for index, item in enumerate(data):
             self[index] = item
+        self._updating = False
     
     def __len__(self):
         return self._len
 
+
 class _PHelperColLabels(_PColsMixin, _PHelperData):
-    """
-    Encapsulates the column labels of a multi-column widget
-    with _PHelper.
-    """
+    # Encapsulates the column labels of a multi-column widget
+    # with _PHelper.
     
     def __init__(self, helper, labels=None):
         self._helper = helper
@@ -85,31 +88,35 @@ class _PHelperColLabels(_PColsMixin, _PHelperData):
         self._set_readonly(index, value.readonly)
     
     def _get_label(self, index):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def _set_label(self, index, label):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def _set_width(self, index, width):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def _set_align(self, index, align):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def _set_readonly(self, index, readonly):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
 
+
 class _PShift(object):
-    """
-    Mixin class to provide 'index-shift' functionality for
-    implementation of sequence classes. Uses a dict with
-    index values as keys as the underlying data store.
-    """
+    # Mixin class to provide 'index-shift' functionality for
+    # implementation of sequence classes. Uses a dict with
+    # index values as keys as the underlying data store.
     
     def __init__(self):
         self._items = {}
@@ -118,11 +125,11 @@ class _PShift(object):
         return len(self._items)
     
     def _shiftright(self, index):
-        """ Move items at index and above 'up' one index (to be called
-        before insertion at index). Note that this only re-indexes the
-        items dict -- if there are any other side effects that should
-        happen as a result of re-indexing, this method must be overridden
-        to implement them. """
+        # Move items at index and above 'up' one index (to be called
+        # before insertion at index). Note that this only re-indexes the
+        # items dict -- if there are any other side effects that should
+        # happen as a result of re-indexing, this method must be overridden
+        # to implement them.
         
         n = len(self._items)
         if index < n:
@@ -134,11 +141,11 @@ class _PShift(object):
             self._items[index] = None
     
     def _shiftleft(self, index):
-        """ Move items above index in the list 'down' one index (to be
-        called after a deletion at index). Note that this only re-indexes
-        the items dict -- if there are any other side effects that should
-        happen as a result of re-indexing, this method must be overridden
-        to implement them. """
+        # Move items above index in the list 'down' one index (to be
+        # called after a deletion at index). Note that this only re-indexes
+        # the items dict -- if there are any other side effects that should
+        # happen as a result of re-indexing, this method must be overridden
+        # to implement them.
         
         n = len(self._items)
         if index < n:
@@ -149,7 +156,8 @@ class _PShift(object):
             del self._items[n]
 
 class _PHelper(_PShift, SortMixin, abstractlist):
-    """
+    """Make a widget look like a list of child items.
+    
     Abstract class to provide common methods to make a widget look
     like a list of 'child' items. Each 'child' item should be a mixin
     of a _PHelperItem (see above) and either a child widget (such as a
@@ -180,65 +188,59 @@ class _PHelper(_PShift, SortMixin, abstractlist):
             self.labels = []
         abstractlist.__init__(self, data)
     
-    def _newitem(self, index, value):
-        """
-        Derived classes can override to change the signature of
-        the item class or substitute another callable that returns
-        an item instance.
-        """
+    def set_header_font(self, font_name=None, font_size=None,
+            bold=None, italic=None):
         
+        self._set_header_font_object(*font_args(
+            self, font_name, font_size, bold, italic))
+    
+    def _set_header_font_object(self, font_name, font_size, bold, italic):
+        """Placeholder for derived classes to implement.
+        """
+        raise NotImplementedError
+    
+    def _newitem(self, index, value):
+        # Derived classes can override to change the signature of
+        # the item class or substitute another callable that returns
+        # an item instance.
         return self.itemclass(self, index, value)
     
     def _helperadd(self, index, value):
-        """
-        Derived classes should override to implement side effects of
-        insertion (if they cannot be handled completely by the helper
-        item's constructor).
-        """
-        
+        # Derived classes should override to implement side effects of
+        # insertion (if they cannot be handled completely by the helper
+        # item's constructor).
         pass
     
     def _helperdel(self, index, item):
-        """
-        Derived classes should override to implement side effects of
-        deletion (if they cannot be handled completely by the helper item's
-        destructor).
-        """
-        
+        # Derived classes should override to implement side effects of
+        # deletion (if they cannot be handled completely by the helper item's
+        # destructor).
         pass
     
     def _get(self, index):
-        """
-        Return the helper item at index. Note that we assume that each
-        item stored in the items dict is a _PHelperItem or derived from it.
-        """
-        
+        # Return the helper item at index. Note that we assume that each
+        # item stored in the items dict is a _PHelperItem or derived from it.
         return self._items[index]
     
     def _set(self, index, value):
-        """
-        Update helper item at index with new value. Note that this method
-        should never be called with an index out of range (meaning that the
-        index will always be in the items dict); adding a new item should always
-        be done using the _add method.
-        """
-        
+        # Update helper item at index with new value. Note that this method
+        # should never be called with an index out of range (meaning that the
+        # index will always be in the items dict); adding a new item should
+        # always be done using the _add method.
         self._items[index]._update(value)
     
     def _add(self, index, value):
-        """ Insert or append a helper item wrapped around value. """
+        # Insert or append a helper item wrapped around value.
         
         self._shiftright(index)
         self._helperadd(index, value)
         self._items[index] = self._newitem(index, value)
     
     def _del(self, index):
-        """
-        Delete the helper item at index. Note that we allow side effects
-        to take place with deletion by calling _helperdel, but unless that
-        method explicitly does so, the object wrapped by the deleted helper
-        item is not deleted unless the helper item holds the only reference.
-        """
+        # Delete the helper item at index. Note that we allow side effects
+        # to take place with deletion by calling _helperdel, but unless that
+        # method explicitly does so, the object wrapped by the deleted helper
+        # item is not deleted unless the helper item holds the only reference.
         
         item = self._items[index]
         del self._items[index]
@@ -272,10 +274,11 @@ class _PHelper(_PShift, SortMixin, abstractlist):
         else:
             raise TypeError, "List index must be an int."
 
+
 class _PRowHelperItem(_PColsMixin, _PHelperItem):
-    """
-    Helper item that exposes a sequence of columns, intended
-    to map to a row in a multi-column widget which does *not*
+    """Helper item that exposes a sequence of columns.
+    
+    Intended to map to a row in a multi-column widget which does *not*
     have nested tree functionality, such as a table.
     """
     
@@ -283,10 +286,11 @@ class _PRowHelperItem(_PColsMixin, _PHelperItem):
         self._helper = helper
         _PHelperItem.__init__(self, index, data)
 
+
 class _PRowHelper(_PHelper):
-    """
-    Helper class for list or table-type widgets where each
-    row has a helper item, but no nested tree structure.
+    """Helper class for list or table-type widgets.
+     
+    Assumes that each row has a helper item, but no nested tree structure.
     """
     
     def _helperadd(self, index, value):
@@ -296,19 +300,23 @@ class _PRowHelper(_PHelper):
         self._remove_row(index)
     
     def _insert_row(self, index):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def _remove_row(self, index):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def rowcount(self):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def current_row(self):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def current_item(self):
@@ -318,18 +326,25 @@ class _PRowHelper(_PHelper):
         else:
             return self._items[r]
 
+
 class _PTreeHelper(_PHelper):
-    """
-    Helper class for tree-type widgets, provides methods to
-    easily manage the entire tree with this object as root.
+    """Helper class for tree-type widgets.
+    
+    Provides methods to easily manage a tree with this object as root.
     """
     
     def current_item(self):
-        """ Derived classes must override to return the currently selected helper item. """
+        """Return the currently selected helper item.
+        
+        Derived classes must override to implement.
+        """
         raise NotImplementedError
     
     def set_current_item(self, item):
-        """ Placeholder for derived classes to implement. """
+        """Set the currently selected helper item.
+        
+        Derived classes must override to implement.
+        """
         raise NotImplementedError
     
     def current_index(self):
@@ -341,18 +356,19 @@ class _PTreeHelper(_PHelper):
     def set_current_index(self, index):
         self.set_current_item(self._items[index])
 
+
 class _PTreeHelperItemCols(_PColsMixin, _PHelperData):
-    """
-    Class to expose columns of a tree view node as a
-    sequence.
+    """Expose columns of a tree view node as a sequence.
     """
     
     def __init__(self, helperitem, data=None):
         self._helperitem = helperitem
         _PHelperData.__init__(self, data)
 
+
 class _PTreeHelperItem(_PHelperItem, _PTreeHelper):
-    """
+    """Tree helper item that is also the root of its own subtree.
+    
     Helper item that appears as its own nested sequence of
     helper items, intended to map to a tree-style widget such
     as a list view item. The data parameter must be a 2-tuple,
