@@ -2,7 +2,7 @@
 """
 Module LISTVIEW -- GUI Tree/List View Widgets
 Sub-Package GUI.WIDGETS of Package PLIB -- Python GUI Framework
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -15,8 +15,10 @@ import sys
 from plib.gui.defs import *
 from plib.gui._widgets import _control, _helpers, _signal
 
+
 class PListViewItemCols(_helpers._PTreeHelperItemCols):
-    """ Exposes columns of a list view item as a sequence. """
+    """Exposes columns of a list view item as a sequence.
+    """
     
     def _get_listviewitem(self):
         return self._helperitem
@@ -29,10 +31,12 @@ class PListViewItemCols(_helpers._PTreeHelperItemCols):
     def _set_data(self, index, value):
         self.listviewitem._set_col(index, value)
 
+
 class PListViewItemBase(_helpers._PTreeHelperItem):
-    """
-    List view item that looks like a sequence of tuples of the
-    form (column-values, [list of child items]). The column-value
+    """List view item that looks like a sequence of 2-tuples.
+    
+    Each 2-tuple in the sequence is of the form
+    (column-values, [list of child items]). The column-values
     is a list of strings; if the list view has only one column,
     the list has a single element.
     """
@@ -41,12 +45,14 @@ class PListViewItemBase(_helpers._PTreeHelperItem):
     
     def __init__(self, parent, index, data=None):
         self._parent = parent
+        self._depth = 0
         self._listview = self._listview_from_parent(parent)
         _helpers._PTreeHelperItem.__init__(self, parent, index, data)
     
     def _listview_from_parent(self, p):
         while not isinstance(p, self.listviewclass):
             p = p._parent
+            self._depth += 1
         return p
     
     def _get_listview(self):
@@ -58,17 +64,21 @@ class PListViewItemBase(_helpers._PTreeHelperItem):
         return self.listview.colcount()
     
     def _get_col(self, col):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def _set_col(self, col, value):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
     
     def expand(self):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
 
+
 class PListViewLabelsBase(_helpers._PHelperColLabels):
     
     def _get_listview(self):
@@ -76,13 +86,12 @@ class PListViewLabelsBase(_helpers._PHelperColLabels):
     
     listview = property(_get_listview)
 
+
 class PListViewMixin(_helpers._PTreeHelper):
-    """
-    Mixin class for PListViewBase and PListBoxBase, to implement
-    common behaviors.
-    """
+    """Mixin class for PListViewBase and PListBoxBase.
     
-    signal = SIGNAL_LISTVIEWCHANGED
+    Implements common behaviors.
+    """
     
     def __init__(self, parent, labels=None, data=None):
         self._parent = parent
@@ -91,24 +100,30 @@ class PListViewMixin(_helpers._PTreeHelper):
         _helpers._PTreeHelper.__init__(self, labels, data)
     
     def colcount(self):
-        """ Placeholder for derived classes to implement. """
+        """Placeholder for derived classes to implement.
+        """
         raise NotImplementedError
 
-class PListViewBase(PListViewMixin, _signal._PNotifyControl):
-    """
-    List view that looks like a list of child list view items.
+
+class PListViewBase(_signal._PNotifyControl, PListViewMixin):
+    """List view that looks like a list of child list view items.
     """
     
+    signal = SIGNAL_LISTSELECTED
+    
     def __init__(self, parent, labels=None, data=None, target=None):
         PListViewMixin.__init__(self, parent, labels, data)
         _signal._PNotifyControl.__init__(self, target)
 
-class PListBoxBase(PListViewMixin, _control._PDialogControl):
-    """
-    List view specialized for use as a dialog control. Includes
-    convenience methods to handle common special cases.
+
+class PListBoxBase(_control._PDialogControl, PListViewMixin):
+    """List view specialized for use as a dialog control.
+    
+    Includes convenience methods to handle common special cases.
     """
     
+    signal = SIGNAL_LISTSELECTED
+    
     def __init__(self, parent, labels=None, data=None, target=None, geometry=None):
         PListViewMixin.__init__(self, parent, labels, data)
         _control._PDialogControl.__init__(self, target, geometry)
@@ -116,21 +131,33 @@ class PListBoxBase(PListViewMixin, _control._PDialogControl):
     if sys.version_info < (2, 4):
         
         def extend_flat(self, items):
-            """ Adds list of items to list box; assumes no children, but multiple columns. """
+            """Adds list of multi-column items to list box.
+            
+            Assumes no children, but multiple columns.
+            """
             for item in items:
                 self.append((item, []))
         
         def extend_list(self, items):
-            """ Adds list of items to list box; assumes a single column list box with no children. """
+            """Adds list of strings to list box.
+            
+            Assumes a single column list box with no children.
+            """
             for item in items:
                 self.append(([item], []))
     
     else:
         
         def extend_flat(self, items):
-            """ Adds list of items to list box; assumes no children, but multiple columns. """
+            """Adds list of multi-column items to list box.
+            
+            Assumes no children, but multiple columns.
+            """
             self.extend((item, []) for item in items)
         
         def extend_list(self, items):
-            """ Adds list of items to list box; assumes a single column list box with no children. """
+            """Adds list of strings to list box.
+            
+            Assumes a single column list box with no children.
+            """
             self.extend(([item], []) for item in items)
