@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 """
 Sub-Package INI of Package PLIB -- Python INI Objects
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -10,7 +10,7 @@ The idea behind these classes is to provide a platform-independent system for
 handling INI options.
 
 The PIniInterface class handles platform-specific stuff so the other classes
-don't have to worry about it; they just call the generic methods of that class.
+don't have to worry about it; they just call that class's generic methods.
 
 The PIniFileOption, PIniFileSection, and PIniFileBase classes implement the
 platform-independent option-handling system. They can be used on their own,
@@ -36,10 +36,11 @@ each of which also stores a list of tuples (one per option), thusly:
 where sectionname is the name of the INI section, optionname is the name of
 the INI option, optiontype is one of the three data type flags below,
 optiondefault is the default value of the option, and optionget and optionset
-are functions that read and write the app variable for the option; or, alternatively,
-attrname is a string giving the name of the attribute on the INI file object to
-be used to store the value of the option. If neither of these is present, the
-default attribute name <sectionname>_<optionname> is used to store the value.
+are functions that read and write the app variable for the option; or,
+attrname is a string giving the name of the attribute on the INI file object
+to be used to store the value of the option. If neither of these is present,
+the default attribute name <sectionname>_<optionname> is used to store the
+value.
 
 The get and set functions, if used, should be of the following form:
 
@@ -74,16 +75,17 @@ elif os.name == "nt":
     # (the iniext global can also be changed to append something to
     # MyAppName but this is very rare)
     from _winreg import *
-    regtypes = {}
-    regtypes[INI_INTEGER] = REG_DWORD
-    regtypes[INI_BOOLEAN] = REG_DWORD
-    regtypes[INI_STRING] = REG_SZ
+    regtypes = {
+        INI_INTEGER: REG_DWORD,
+        INI_BOOLEAN: REG_DWORD,
+        INI_STRING: REG_SZ }
     inipath = "Software"
     iniext = ""
 
+
 class PIniInterface(object):
-    """
-    Low-level interface with the OS mechanisms for storing preferences.
+    """Low-level interface with the OS mechanisms for storing preferences.
+    
     The intent is to hide all OS-dependent code in this class so the
     other classes see a single consistent API.
     """
@@ -175,7 +177,10 @@ class PIniInterface(object):
                 return adefault
         
         elif self.openkey is not None:
-            retval = QueryValueEx(self.openkey, oname)[0]
+            try:
+                retval = QueryValueEx(self.openkey, oname)[0]
+            except WindowsError:
+                retval = adefault
             if vtype == INI_INTEGER:
                 return int(retval)
             elif vtype == INI_BOOLEAN:
@@ -188,8 +193,8 @@ class PIniInterface(object):
     
     def writevalue(self, oname, vtype, avalue):
         if self.parser is not None:
-            # Dunno why ConfigParser doesn't have setint, setboolean corresponding
-            # to getint, getboolean above, but oh well...
+            # Dunno why ConfigParser doesn't have setint, setboolean
+            # corresponding to getint, getboolean above, but oh well...
             if vtype == INI_BOOLEAN:
                 avalue = {False: self.falsestr, True: self.truestr}[avalue]
             else:
@@ -199,12 +204,13 @@ class PIniInterface(object):
         elif self.openkey is not None:
             SetValueEx(self.openkey, oname, 0, regtypes[vtype], str(avalue))
 
+
 class PIniFileOption(object):
-    """
-    OS-independent class for a single preference option. Option must have a
-    name, a data type, a value, and a default value (used if option is not
-    yet read or not present in the preferences file). Expects to be given a
-    PIniFileInterface instance to read or write its value.
+    """OS-independent class for a single preference option.
+    
+    Option must have a name, data type, value, and default (used if option is
+    not yet read or not present in the preferences file). Expects to be given
+    a PIniFileInterface instance to read or write its value.
     """
     
     def __init__(self, oname, vtype, adefault):
@@ -227,10 +233,11 @@ class PIniFileOption(object):
     def write(self, intf):
         intf.writevalue(self.optname, self.opttype, self.optvalue)
 
+
 class PIniFileSection(object):
-    """
-    OS-independent class for a preferences section containing multiple
-    options. Section has a name and a dictionary of names (keys) vs.
+    """OS-independent class for preferences section with multiple options.
+    
+    Section has a name and a dictionary of names (keys) vs.
     PIniFileOption instances. Expects to be given a PIniFileInterface
     instance to read or write its options.
     """
@@ -268,13 +275,14 @@ class PIniFileSection(object):
             self.options[oname].write(intf)
         intf.closesection(self.sectname)
 
+
 class PIniFileBase(object):
-    """
-    OS-independent base class for a preferences file. Has a name (which
-    denotes the entire file), a dictionary of section names (keys) vs.
-    PIniFileSection instances, and a PIniInterface to use for the actual
-    reading/writing of preferences (the actual read/write mechanics walk
-    the section/option hierarchy for simplicity of coding).
+    """OS-independent base class for a preferences file.
+    
+    Has a name (which denotes the entire file), a dictionary of section
+    names (keys) vs. PIniFileSection instances, and a PIniInterface to use
+    for the actual reading/writing of preferences (the actual read/write
+    mechanics walk the section/option hierarchy for simplicity of coding).
     
     Note this base class is factored out from PIniFile below to allow
     different methods of accessing the INI file structure. PIniFile uses
@@ -323,10 +331,11 @@ class PIniFileBase(object):
             self.sections[sname].write(self.intf)
         self.intf.shutdown(self.ininame, True)
 
+
 class PIniFile(PIniFileBase):
-    """
-    OS-independent class for an actual preferences file; keeps a
-    dictionary of (section name, option name) tuples vs. get/set
+    """OS-independent class for an actual preferences file.
+    
+    Keeps a dictionary of (section name, option name) tuples vs. get/set
     functions to link the preferences to program actions. Provides
     three API methods for applications to use, as noted in the
     module documentation above: buildini, readini, and writeini.
@@ -374,7 +383,8 @@ class PIniFile(PIniFileBase):
                 self.vardict[(sect[0], opt[0])] = entry
     
     def readini(self):
-        """ Called to read preferences from file and feed them to the program. """
+        """Read preferences from file and feed them to the program.
+        """
         self.readvalues()
         for key in self.vardict.iterkeys():
             entry = self.vardict[key]
@@ -385,7 +395,8 @@ class PIniFile(PIniFileBase):
                 setattr(self, entry, value)
     
     def writeini(self):
-        """ Called to feed preferences data from the program and write to file. """
+        """Get preferences data from the program and write to file.
+        """
         for key in self.vardict.iterkeys():
             entry = self.vardict[key]
             if isinstance(entry, tuple):
