@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 """
 Sub-Package STDLIB of Package PLIB -- Python Standard Library Extensions
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -30,30 +30,14 @@ variables pythonpath, plibpath, binpath, sharepath --
     (the latter two are where plib's scripts and example programs will
     have been installed)
 
-functions any, all -- a little different from the functions built in
-    to Python 2.5, these allow an optional predicate
+function closure -- a reference to ``plib.stdlib.func.partial``,
+    provided for convenience since the name is more indicative of
+    the usage to some
 
-function closure -- similar to functools.partial from Python 2.5; returns
-    a callable object that returns the same result as calling the passed
-    function with the passed arguments (but does not allow additional
-    arguments with the actual call, as functools.partial does)
-
-function dotted_import -- wraps the __import__ builtin so that it
-    returns the 'innermost' module in a dotted name; the equivalent
-    of 'import x.y.z'
-
-function dotted_from_import -- dotted_import, plus a getattr call to
-    retrieve the named attribute from the dotted module; the equivalent
-    of 'from x.y.z import a' (except that module a of package x.y.z
-    will not be returned unless it has already been imported into the
-    namespace of x.y.z)
-
-function first -- returns first item in iterable, or default if empty.
+function gcd -- fast implementation of Euclid's GCD algorithm.
 
 function inrange -- returns index forced to be >= low, <= high.
 
-function inverted -- returns the inverse of a mapping.
-
 function iterfile --
 
 Returns a generator that can be used in place of "for line in file"
@@ -68,7 +52,7 @@ fixes it. Note that the issue can also arise, as the blog entry notes,
 with line-oriented network protocols, which means any time you are
 using a "file-like object" derived from a socket.
 
-function last -- returns last item in iterable, or default if empty.
+function lcm -- fast implementation of LCM based on GCD above.
 
 function normalize -- returns index with negative values normalized
     relative to count.
@@ -78,13 +62,12 @@ function normalize_slice -- returns a list of indexes "covered" by a
     index to which a zero-length slice refers (normalized relative
     to count).
 
+function prod -- returns the product of all items in an iterable.
+
 function slice_len -- returns the "length" of a slice (meaning the
     number of indexes that would be affected if it were used as an
     index into a sequence).
 
-function sorted -- backports the builtin to Python versions < 2.4. No
-    reverse functionality for earlier versions.
-
 functions strtobool, strtodate -- self-explanatory.
 
 function cached_property (and auxiliary class CachedProperty) --
@@ -119,57 +102,35 @@ overhead of the property altogether after the first access,
 the additional overhead of a Python class instance is
 negligible compared to the (presumably expensive) initial
 calculation of the property value.
+
+function upgrade_builtins -- convenience function to ensure
+    that function names from more recent Python versions are
+    in the builtin namespace even in earlier versions. See
+    the function docstring for details.
 """
 
 import sys
-import os
 import datetime
-import glob
-from itertools import ifilter, ifilterfalse
+from operator import mul
 
 from plib.utils import ModuleProxy
 
 from _paths import *
-from SortMixin import SortMixin
 
-def any(iterable, predicate=bool):
-    """ Return True if any element of iterable satisfies predicate. """
-    for element in ifilter(predicate, iterable):
-        return True
-    return False
+from plib.stdlib.func import partial as closure
 
-def all(iterable, predicate=bool):
-    """ Return True only if all elements of iterable satisfy predicate. """
-    for element in ifilterfalse(predicate, iterable):
-        return False
-    return True
 
-def closure(fn, *args, **kwargs):
-    """ Return a no-parameter callable equivalent to fn(*args, **kwargs). """
-    def f():
-        return fn(*args, **kwargs)
-    return f
+def gcd(x, y):
+    """Return greatest common divisor of x, y.
+    """
+    while x:
+        x, y = y % x, x
+    return y
 
-def dotted_import(name):
-    """ Returns the module pointed to by import x.y.z. """
-    mod = __import__(name)
-    components = name.split('.')
-    for comp in components[1:]:
-        mod = getattr(mod, comp)
-    return mod
-
-def dotted_from_import(modname, attrname):
-    """ Returns the module attribute pointed to by from x.y.z import a. """
-    return getattr(dotted_import(modname), attrname)
-
-def first(iterable, default=None):
-    """ Return first item in iterable, or default if empty. """
-    for item in iterable:
-        return item
-    return default
 
 def inrange(index, v1, v2):
-    """ Force index to be within the range v1 to v2, and return result. """
+    """Force index to be within the range v1 to v2, and return result.
+    """
     if v1 < v2:
         low = v1
         high = v2
@@ -178,74 +139,35 @@ def inrange(index, v1, v2):
         high = v1
     return min(max(index, low), high)
 
-if sys.version_info < (2, 4):
-    
-    def inverted(mapping, keylist=None):
-        """
-        Return a dict that is the inverse of the given mapping. The
-        optional argument keylist limits the keys that are inverted.
-        
-        This version is for Python 2.3 and earlier and uses a for
-        loop to avoid making an extra in-memory copy of the mapping
-        (for the intermediate sequence of inverted value, key tuples)
-        as a list comprehension or built-in function such as zip
-        would do. This implementation is significantly slower than
-        the Python 2.4 and later implementation using generator
-        expressions.
-        """
-        
-        result = mapping.__class__()
-        if keylist is not None:
-            for key in keylist:
-                result[mapping[key]] = key
-        else:
-            for key, value in mapping.iteritems():
-                result[value] = key
-        return result
-
-else:
-    
-    def inverted(mapping, keylist=None):
-        """
-        Return a dict that is the inverse of the given mapping. The
-        optional argument keylist limits the keys that are inverted.
-        
-        Note that we use a generator expression and iteritems to avoid
-        making an extra in-memory copy of the mapping (for the intermediate
-        sequence of inverted value, key tuples). This version only works
-        for Python 2.4 and later.
-        """
-        
-        if keylist is not None:
-            return mapping.__class__((mapping[key], key) for key in keylist)
-        else:
-            return mapping.__class__((value, key) for key, value in mapping.iteritems())
 
 def iterfile(f):
-    """
-    Returns a generator that yields lines from a file as "for line in file"
-    does, but avoids potential problems with buffering. Use as
+    """Return a generator that yields lines from a file.
     
-    for line in iterfile(file):
+    This works like "for line in file" does, but avoids potential
+    problems with buffering. Use as::
+    
+        for line in iterfile(file):
     """
+    
     while 1:
-        line = f.readline() # this guarantees getting each line when it's ready
+        line = f.readline() # guarantees getting each line when it's ready
         if not line:
             return # we've reached EOF
         yield line
 
-def last(iterable, default=None):
-    """ Return last item in iterable, or default if empty. """
-    result = default
-    for item in iterable:
-        result = item
-    return result
+
+def lcm(x, y):
+    """Return least common multiple of x, y.
+    """
+    if (x == 0) and (y == 0):
+        return 0
+    return x * y / gcd(x, y)
+
 
 def normalize(count, index):
-    """
-    Return index value with negative indexes converted to indexes
-    relative to endpoint. Index values out of range after conversion
-    will raise IndexError.
+    """Return index with negative values normalized to count.
+    
+    Index values out of range after conversion will raise IndexError.
     """
     
     if index < 0:
@@ -256,18 +178,20 @@ def normalize(count, index):
         raise IndexError, "sequence index out of range"
     return result
 
+
 def normalize_slice(count, index):
-    """
+    """Return slice index normalized to count.
+    
     Return one of the following, depending on the type of slice index:
     
-    -- For a non-empty slice (e.g., [x:y] where x != y), return a list of
-       indexes to be affected;
+    - For a non-empty slice (e.g., [x:y] where x != y), return a list of
+      indexes to be affected;
     
-    -- For an empty slice (e.g., [x:x]), return the slice location (x)
-       as an int.
+    - For an empty slice (e.g., [x:x]), return the slice location (x)
+      as an int.
     
-    -- For extended slices, if start and stop are the same, return an
-       empty list; otherwise treat as a non-empty slice.
+    - For extended slices, if start and stop are the same, return an
+      empty list; otherwise treat as a non-empty slice.
     
     The index(es) returned will be normalized relative to count, and indexes
     out of range after normalization will be truncated to range(0, count + 1).
@@ -321,13 +245,21 @@ def normalize_slice(count, index):
             stop = inrange(stop, 0, count)
         return range(start, stop, step)
 
-def slice_len(s):
+
+def prod(iterable):
+    """Return the product of all items in iterable.
     """
-    Return the number of indexes referenced by a slice. Note that
-    we do not normalize the slice indexes, since we don't have a
-    sequence length to reference them to. Also note that, because
-    of this, we have to return None if there is no stop value, or
-    if start or stop are negative.
+    
+    return reduce(mul, iterable, 1)
+
+
+def slice_len(s):
+    """Return the number of indexes referenced by a slice.
+    
+    Note that we do not normalize the slice indexes, since we don't
+    have a sequence length to reference them to. Also note that,
+    because of this, we have to return None if there is no stop value,
+    or if start or stop are negative.
     """
     
     if s.start is None:
@@ -352,53 +284,27 @@ def slice_len(s):
     else:
         return (stop - start) // step
 
-if sys.version < 2.4:
-    
-    class _sorted_helper(SortMixin, list): pass
-    
-    def sorted(iterable, sortfunc=None, key=None):
-        if isinstance(iterable, list):
-            result = iterable[:]
-            if sortfunc is None:
-                if key is not None:
-                    sortfunc = lambda x, y: cmp(key(x), key(y))
-            if sortfunc is None:
-                result.sort()
-            else:
-                result.sort(sortfunc)
-            return result
-        else:
-            result = _sorted_helper()
-            for item in iterable:
-                result.insert_sorted(item, sortfunc, key)
-            return list(result)
-
-else:
-    
-    from __builtin__ import sorted
 
 def strtobool(s):
-    """
-    Return bool from string s interpreting s as a 'Python value string'.
+    """Return bool from string s interpreting s as a 'Python value string'.
+    
     Return None if s is not 'True' or 'False'.
     """
     
-    if s == 'True':
-        return True
-    if s == 'False':
-        return False
-    return None
+    return {'False': False, 'True': True}.get(s, None)
+
 
 def strtodate(s):
-    """ Return date object from string s formatted as date.__str__ would return """
+    """Return date object from string formatted as date.__str__ would return.
+    """
     
-    dateargs = s.split("-")
-    return datetime.date(int(dateargs[0]), int(dateargs[1]), int(dateargs[2]))
+    return datetime.date(*map(int, s.split('-')))
+
 
 class CachedProperty(object):
-    """
-    Non-data descriptor class for cached property. The
-    expected typical use case is to be called from the
+    """Non-data descriptor class for cached property.
+    
+    The expected typical use case is to be called from the
     cached_property function, which generates the name of
     the property automatically, but the class can also be
     instantiated directly with an explicit name supplied.
@@ -416,11 +322,12 @@ class CachedProperty(object):
         setattr(obj, self.aname, result)
         return result
 
+
 def cached_property(fget, doc=None):
-    """
-    Function to return cached property instance. We need
-    this as a wrapper to supply the name of the property by
-    magic rather than force the user to enter it by hand;
+    """Function to return cached property instance.
+    
+    We need this as a wrapper to supply the name of the property
+    by magic rather than force the user to enter it by hand;
     this is done by looking up the name of the fget function
     (which also allows this function to be used as a decorator
     and have the intended effect).
@@ -430,43 +337,112 @@ def cached_property(fget, doc=None):
         doc = fget.__doc__
     return CachedProperty(fget.__name__, fget, doc)
 
+
+### Convenience function to upgrade builtin namespace ###
+
+BUILTIN_NAMES = [
+    'all',
+    'any',
+    'reversed',
+    'sorted'
+]
+
+EXTRA_NAMES = [
+    'first',
+    'inverted',
+    'last'
+]
+
+# This ensures we don't run the upgrade multiple times
+_upgraded = False
+
+
+def upgrade_builtins():
+    """Upgrades __builtin__ namespace in earlier Python versions.
+    
+    This function is provided as a convenience for easier coding when using
+    functions that are builtins in later Python versions, but which would
+    normally have to be imported from plib.stdlib in earlier versions. For
+    example, without this function, you would have to say::
+    
+        try:
+            any
+        except NameError:
+            from plib.stdlib import any
+    
+    in order to ensure that the ``any`` function was accessible in your
+    module. If you were using multiple such functions, this stanza would
+    quickly become large and messy.
+    
+    An alternative to this function would be to put a similar stanza in the
+    plib.stdlib code, so that such functions would always appear in the
+    plib.stdlib namespace, whether they were builtins in the running Python
+    version or not. However, this would still require you to say::
+    
+        from plib.stdlib import <builtin1>, <builtin2>, ...
+    
+    in your module, meaning that you would have to keep track of which
+    builtins you were using module by module (or else use from import *,
+    which is considered highly undesirable by all right-thinking Python
+    programmers). Since the whole point of having built-in functions is to
+    not have to do such things, this seems less than optimal.
+    
+    This function solves the problem as best it can be solved without major
+    magical hackery at the time of importing plib.stdlib--which I may decide
+    to try in future, so you have been warned. :-) But at present, this is
+    the intended usage::
+    
+        # Do this somewhere in your code, usually at the end of imports
+        from plib.stdlib import upgrade_builtins
+        upgrade_builtins(__name__)
+    
+    This will add equivalent functions to the built-in namespace for all
+    builtins from the following list that are not already present in the
+    running version of Python. In addition, it will add the plib-specific
+    "builtins" listed below, which are not provided by default in any
+    current Python version, but which are so general and useful that IMHO
+    they ought to be. :-) You only need to do this once, anywhere in your
+    code, and the upgraded builtins will be available for the life of that
+    invocation of the interpreter.
+    
+    Builtins currently provided:
+    
+    - all
+    - any
+    - reversed
+    - sorted
+    
+    Additional plib-specific "builtins":
+    
+    - first
+    - inverted
+    - last
+    """
+    
+    global _upgraded
+    if not _upgraded:
+        import __builtin__
+        for builtin_name in BUILTIN_NAMES:
+            if not hasattr(__builtin__, builtin_name):
+                import _builtins
+                setattr(__builtin__, builtin_name,
+                    getattr(_builtins, builtin_name))
+        for extra_name in EXTRA_NAMES:
+            import _extras
+            setattr(__builtin__, extra_name, getattr(_extras, extra_name))
+        _upgraded = True
+
 # *************** end of 'internal' functions for this module ***************
 
 # Now we do the ModuleProxy magic to make classes in
 # our sub-modules appear in our namespace
 
-# Helper function to actually do the import when the
-# module attribute is accessed; note that it assumes
-# that the function returns the actual class defined
-# in the module, not the module itself, and it assumes
-# that the class will have the same name as the module
+excludes = ['_builtins', '_extras',
+    '_defaultdict', '_namedtuple',
+    'coll', 'decotools', 'func', 'imp', 'options']
 
-def module_helper(modname):
-    def f():
-        result = __import__(modname, globals(), locals())
-        return getattr(result, modname)
-    f.__name__ = "%s_helper" % modname
-    return f
-
-# Generate a dictionary of module names and classes
-# in our sub-package; note that here (unlike in the
-# CLASSES sub-package) we have a few modules we want
-# to exclude because we actually want them to appear
-# in our namespace as modules, not classes
-
-excludes = ['__init__', 'coll', 'options']
-module_dict = {}
-for pathname in __path__:
-    for filename in glob.glob(os.path.join(pathname, "*.py")):
-        modname = os.path.splitext(os.path.basename(filename))[0]
-        if modname not in excludes:
-            module_dict[modname] = module_helper(modname)
-
-# Now do the module proxy -- see comments in the CLASSES
-# sub-package for caveats when doing this
-
-ModuleProxy(__name__, module_dict)
+ModuleProxy(__name__).init_proxy(__name__, __path__, globals(), locals(),
+    excludes=excludes)
 
 # Now clean up our namespace
-del glob, ModuleProxy, module_helper, module_dict, \
-    pathname, filename, modname
+del ModuleProxy
