@@ -2,7 +2,7 @@
 """
 Module SerialClientMixin
 Sub-Package STDLIB.IO.ASYNC of Package PLIB
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -13,12 +13,12 @@ This module contains the asynchronous SerialClientMixin class.
 from plib.stdlib.io.serial import BaseClient
 from plib.stdlib.io.async import ClientMixin
 
+
 class SerialClientMixin(BaseClient, ClientMixin):
-    """
-    Mixin class to provide basic asynchronous serial client
-    functionality. Call the ``client_communicate method`` to
-    open a serial device and send data; override the
-    ``process_data`` method to do something with the reply.
-    """
+    """Mixin class for async serial client.
     
+    Call the ``client_communicate method`` to open a serial
+    device and send data; override the ``process_data`` method
+    to do something with the reply.
+    """
     pass
