@@ -2,7 +2,7 @@
 """
 Module ConnectMixin
 Sub-Package STDLIB.IO.SOCKET of Package PLIB
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -12,17 +12,16 @@ This module contains the socket ConnectMixin class.
 
 import socket
 
+
 class ConnectMixin(object):
-    """
-    Mixin class to add socket connect functionality. Usable by both
-    "one-loop" and persistent clients.
+    """Mixin class to add socket connect functionality.
     """
     
     address_family = socket.AF_INET
     socket_type = socket.SOCK_STREAM
     
     def do_connect(self, addr):
-        """ Convenience method to create socket and connect to addr. """
-        
+        """Convenience method to create socket and connect to addr.
+        """
         self.create_socket(self.address_family, self.socket_type)
         self.connect(addr)
