@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 """
 UTILS_MODULEPROXY_TESTMOD.PY -- test module for PLIB.UTILS ModuleProxy tests
-Copyright (C) 2008-2009 by Peter A. Donis
+Copyright (C) 2008-2011 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -20,6 +20,7 @@ _names = {
     'test_var_dynamic': "Dynamic test variable.",
     'test_fn': _test_fn }
 
-ModuleProxy(__name__, _names)
+ModuleProxy(__name__).init_proxy(__name__, None, globals(), locals(),
+    names=_names, autodiscover=False)
 
 del _test_fn, _names
